---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
-- Derivation   : MIT X Window System, V11.R4
--
-- This Ada language binding to the X Window System inherits
-- the MIT copyright. The following copyright must be included
-- in all software utilising this binding.
--
-- Copyright 1990, 1991 J.C. Mahieux and B. Maudry
-- Copyright 1985, 1986, 1987 by the Massachusetts Institute of Technology
--
with X_Types ;

package X_Properties is
    use X_Types ;
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Array_From_Property
    --
    --  <Description>
    --
    --  This generic function provides the mean to convert property data
    --  to the convenient unconstrained array type.
    --
    --  <Generic Parameters>
    --
    --  Object       : Specifies the target element type.
    --  Index        : Specifies the index of the target array type.
    --  Object_Array : Specifies the target array type.
    --
    --  <Parameters>
    --
    --  Data   : Specifies the raw data to convert.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    generic
        type Object is private ;
        type Index is (<>) ;
        type Object_Array is array (Index range <>) of Object ;
    function Array_From_Property ( Data : in Property_Data )
        return Object_Array ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Change_Property
    --
    --  <Description>
    --
    -- This procedure changes the property for a specified window. The
    -- data must be converted from a defined type to the corresponding
    -- property data type.
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --  Window_Id      : Specifies the window.
    --  Property       : Specifies the property atom.
    --  Kind           : Specifies the atom identifier type.
    --  Mode           : Specifies the property mode.
    --  Data           : Specifies the property data.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Window is raised if the window is invalid.
    --  Bad_Atom is raised if the property atom is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Change_Property ( Display_Id : in Display;
                                Window_Id  : in Window;
                                Property   : in Atom;
                                Kind       : in Atom;
                                Mode       : in Property_Mode;
                                Data       : in Property_Data) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Convert_Selection
    --
    --  <Description>
    --
    -- This procedure requests a selection conversion to the specific
    -- target type.
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --  Selection      : Specifies the selection atom.
    --  Target         : Specifies the target atom.
    --  Property       : Specifies the property atom.
    --  Requestor      : Specifies the window Id of the requestor.
    --  Time_Stamp     : Specifies the time.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Atom is raised if an atom is invalid.
    --  Bad_Window is raised if the window is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Convert_Selection ( Display_Id : in Display;
                                  Selection  : in Atom;
                                  Target     : in Atom;
                                  Property   : in Atom;
                                  Requestor  : in Window;
                                  Time_Stamp : in Time);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Delete_Property
    --
    --  <Description>
    --
    -- This procedure deletes a property for the specified window.
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --  Window_Id      : Specifies the window.
    --  Property       : Specifies the property atom to be deleted.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Atom is raised if an atom is invalid.
    --  Bad_Window is raised if the window is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Delete_Property ( Display_Id : in Display;
                                Window_Id  : in Window;
                                Property   : in Atom) ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Fetch_Buffer
    --
    --  <Description>
    --
    --  This generic function fetches data from the specified cut buffer.
    --  The default for From_Buffer makes this equivalent to XFetchBytes in
    --  the MIT C xlib library.
    --
    --  Each element must use no more than 8 bits.
    --  It must be byte-aligned in the array.
    --
    --  <Generic Parameters>
    --
    --  Element     : Specifies the basic component type of data.
    --  Index       : Specifies the maximum size to fetch
    --  Data_Type   : specifies how to understand the stored data.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  From_Buffer : Specifies the buffer to use.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================
    
    generic
        type Element is private ;
        type Index is range <> ;
        type Data_Type is array (Index range <>) of Element ;
        
    function Fetch_Buffer
        ( Display_Id  : in Display;
          From_Buffer : in Buffer_Range := 0) return Data_Type ;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_Atom_Name
    --
    --  <Description>
    --
    -- This function returns the name for the specified atom.
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --  Atom_Id        : Specifies the Atom.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Atom is raised if the atom id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Get_Atom_Name ( Display_Id : in Display;
                             Atom_Id    : in Atom) return String;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_Selection_Owner
    --
    --  <Description>
    --
    -- This function gets the window id of the owner of the selection atom
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --  Selection      : Specifies the selection atom.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Atom is raised if an atom is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Get_Selection_Owner ( Display_Id : in Display;
                                   Selection  : in Atom) return Window;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_Window_Property
    --
    --  <Description>
    --
    -- This procedure returns the atom type and property format for the
    -- specified window. After the data is returned it must be converted
    -- to the appropriate type.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  Window_Id   : Specifies the window.
    --  Property    : Specifies the property atom.
    --  Long_Offset : Specifies the offset in the property where data
    --                will be retrieved.
    --  Long_Length : Specifies the length of the data.
    --  Delete      : Specifies if the prop should be deleted from the window
    --  Req_Type    : Specifies the atom identifier type.
    --  Actual_Type : Returns the actual type of the property.
    --  Actual_Form : Returns the actual format of the property.
    --  Bytes_After : Returns the number of bytes remaining.
    --  Data        : Returns the untyped data.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Window is raised if the window is invalid.
    --  Bad_Atom is raised if the property atom is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Get_Window_Property ( Display_Id  : in Display;
                                    Window_Id   : in Window;
                                    Property    : in Atom;
                                    Long_Offset : in X_Long_Integer;
                                    Long_Length : in X_Long_Integer;
                                    Delete      : in Boolean;
                                    Req_Type    : in Atom;
                                    Actual_Type : out Atom;
                                    Actual_Form : out Property_Format_Type ;
                                    Bytes_After : out X_Long_Integer;
                                    Data        : out Property_Data) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Intern_Atom
    --
    --  <Description>
    --
    -- This function returns an atom for a specified name.
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --  Atom_Name      : Specifies the name associated with the atom.
    --  Only_If_Exists : Specifies that the atom be returned if it exists.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Intern_Atom ( Display_Id     : in Display;
                           Atom_Name      : in String;
                           Only_If_Exists : in Boolean := True)
        return Atom;

    --====================================================================
    --  <Subprogram Name>
    --
    --  List_Properties
    --
    --  <Description>
    --
    -- This function returns a list of properties for the specified window
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --  Window_Id      : Specifies the window.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Window is raised if the window is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function List_Properties
        ( Display_Id : in Display;
          Window_Id  : in Window) return Atom_List ;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Object_From_Property
    --
    --  <Description>
    --
    --  This generic function provides the mean to convert property data
    --  to the convenient object type.
    --
    --  <Generic Parameters>
    --
    --  Object : Specifies the target type.
    --
    --  <Parameters>
    --
    --  Data   : Specifies the raw data to convert.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    generic
        type Object is private ;
    function Object_From_Property ( Data : in Property_Data ) return Object ;
    
    
    --======================================================================
    --  <Subprogram Name>
    --
    --  Rotate_Buffers
    --
    --  <Description>
    --
    --  This procedure rotates the cut buffers such that buffer 0 becomes
    --  buffer n, buffer 1 becomes buffer n+1 mod 8, etc.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  By         : Specifies the rotation.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Rotate_Buffers ( Display_Id : in Display;
                               By         : in Natural);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Rotate_Window_Properties
    --
    --  <Description>
    --
    -- This procedure rotates properties in an atom list.
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --  Window_Id      : Specifies the window.
    --  Properties     : Specifies the list to be rotated.
    --  N_Positions    : Specifies the rotation amount.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if an atom in the list occurs more than once or
    --            no property with that name is defined for the window.
    --  Bad_Atom is raised if an atom is invalid.
    --  Bad_Window is raised if the window is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Rotate_Window_Properties ( Display_Id  : in Display;
                                         Window_Id   : in Window;
                                         Properties  : in Atom_List;
                                         N_Positions : in Natural) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Selection_Owner
    --
    --  <Description>
    --
    -- This procedure changes the owner and last time for the specified
    -- selection.
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --  Selection      : Specifies the selection atom.
    --  Owner          : Specifies the owner of the selection atom.
    --  Time_Stamp     : Specifies the time.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Atom is raised if an atom is invalid.
    --  Bad_Window is raised if the window is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_Selection_Owner ( Display_Id : in Display;
                                    Selection  : in Atom;
                                    Owner      : in Window;
                                    Time_Stamp : in Time) ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Store_Buffer
    --
    --  <Description>
    --
    --  This generic procedure stores data in the specified cut buffer.
    --  The default for To_Buffer makes this equivalent to XStoreBytes in
    --  the MIT C xlib library.
    --
    --  Each element must use no more than 8 bits.
    --  It must be byte-aligned in the array.
    --
    --  <Generic Parameters>
    --
    --  Element     : Specifies the basic component type of data.
    --  Index       : Specifies the maximum size to stored
    --  Data_Type   : specifies the type of the stored data.
    --
    --  <Parameters>
    --
    --  Display_Id      : Specifies the connection to the X Server.
    --  Data            : Specifies the data to store.
    --  To_Buffer       : Specifies the buffer to use.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    generic
        type Element is private ;
        type Index is range <> ;
        type Data_Type is array (Index range <>) of Element ;
        
    procedure Store_Buffer ( Display_Id      : in Display;
                             Data            : in Data_Type;
                             To_Buffer       : in Buffer_Range := 0);

    --====================================================================
    --  <Subprogram Name>
    --
    --  To_Property_Data
    --
    --  <Description>
    --
    --  This generic function provides the mean to convert an object
    --  to the corresponding property data.
    --  Each component inside Object must use the same size, which is
    --  indicated by the generic parameter Format.
    --  Some private types in X_Types may be used inside property data,
    --  using Format_32 :
    --  Atom                 Color_Map             Cursor
    --  Drawable             Font                  Pixmap
    --  Resource             Resource_Gc           Window
    --  X_Id
    --
    --  The following private types in X_Types MUST NOT be used in
    --  property data :
    --  Context              Display               Graphic_Context
    --  Image                Region                Screen
    --  Visual
    --
    --  <Generic Parameters>
    --
    --  Format : Specifies the format of the target property.
    --  Object : Specifies the source type.
    --
    --  <Parameters>
    --
    --  Item   : Specifies the object to convert.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    generic
        Format : in Property_Format_Type ;
        type Object is private ;
    function To_Property_Data ( Item : in Object ) return Property_Data ;


end X_Properties ;
