---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
--
--
-- Package Name - : X_Types
--
-- Title - x_types.ads
--
------------------ Distribution and Copyright -----------------
-- Derivation   : MIT X Window System, V11.R4
--
-- This Ada language binding to the X Window System inherits
-- the MIT copyright. The following copyright must be included
-- in all software utilising this binding.
--
-- Copyright 1990, 1991 J.C. Mahieux and B. Maudry
-- Copyright 1985, 1986, 1987 by the Massachusetts Institute of Technology
--
--
-- This specification is derived from an interface done by :
--     Science Applications International Corporation
--
-- This implementation is made of pure Ada code
-- with no pragma Interface with the C Xlib library.
--
-------------------END-PROLOGUE--------------------------------
--
-- Abstract -
--
-- The package X_Types provides most of the capability for creating,
-- manipulating and using Windows. This package is associated with the other
-- packages providing logical groupings for specific data types or
-- collections of similar operations. In addition, this main package provides
-- the major data types used in the X Window System, all of which are
-- declared private.
-- These are:
--
--    DISPLAY, which is a set of one or more screens driven by a single
--    X Server.
--
--    WINDOW, which is a rectangular area on the screen that lets you
--    view graphical output.
--
--    DRAWABLE, which is either a window or a pixmap to be used as a source
--    or destination in a graphic operation.
--
--
-- X_Types also provides a number of frequently-used data types, such as
-- POINT, RECTANGLE, etc.
--
-- The sets of operations defined by the X Window System fall into the
-- following groups:
--
--    DISPLAY OPERATIONS : These operations act on the type DISPLAY, either
--    establishing or closing the connection to the X Server, or providing
--    information about the display's capabilities. These operations are
--    always the first and last operations used by a X application.
--
--    WINDOW OPERATIONS : These operations allow the application to create,
--    destroy and manipulate windows on an open display. In addition,
--    operations are provided which allow applications to modify window
--    attributes such as background and foreground color, and to translate
--    the coordinates of one window into the coordinate system of another
--    window. Finally, operations are available to allow applications
--    to obtain and manipulate information about windows and their associated
--    properties.
--
--    GRAPHICS RESOURCE OPERATIONS: These operations allow applications to
--    manipulate color maps, pixmaps and graphic contexts. In addition,
--    graphic primitives for drawing points, lines, rectangles, arcs and fil
--    areas are provided. The graphic operations also allow the transfering
--    and copying of areas.
--
--    FONT OPERATIONS : These operations allow applications to obtain fonts
--    for text output.
--
--    TEXT OPERATIONS : These operations allow applications to output text
--    to a window in a variety of styles and fonts.
--
--    CURSOR OPERATIONS : These operations allow applications to create
--    and use various cursors.
--
--    STANDARD PROPERTY OPERATIONS : These operations allow applications to
--    manage the standard properties for common information associated with
--    windows.
--
--    UTILITY OPERATIONS : These operations allow applications to perform
--    a variety of functions such as manipulating regions, obtaining
--    environment defaults and cut-and-paste operations.
--
-- The packages depending on this package are:
--
--     X_COLORS : built around the private type COLOR_MAP and providing
--     operations necessary for manipulating that type.
--
--     X_CURSORS : build around the private type CURSOR, a set of predefined
--     cursor shapes, and the operations for creating and manipulating
--     cursors.
--
--     X_FONTS : built around the private type FONT and providing all the
--     operations necessary for manipulating that type.
--
--     X_GRAPHIC_OUTPUT : build around the private type GRAPHIC_CONTEXT and
--     operations for output to windows.
--
--     X_KEYBOARD : which exports operations for manipulating the keyboard.
--
--     X_PROPERTIES : which exports operations for manipulating atoms
--     and properties.
--
-- In addition, other packages, declared separately, are defined. These
-- are:
--
--    X_DISPLAY_MANAGER : which exports all the operations for manipulating
--    the display and all the associated attributes.
--
--    X_EVENTS : which exports the operations for gathering and manipulating
--    input of events.
--
--    X_EXTENSIONS : which exports all the operations giving access
--    to the available extensions.
--
--    X_RESOURCE_MANAGER : which exports all the data types and operations
--     for manipulating the resource databases.
--
--    X_WINDOW_ACTIONS : which exports all the operations for manipulating
--    the windows and all the associated attributes.
--
--    X_WINDOW_MANAGER : which provides operations, generally unused by X
--    applications for creating and managing Window Managers such as uwm.
--
--    This package depends on the package X_KEY_SYMBOLS which defines all
--    the key symbols constants.
--
-- Information pertaining to X Windows can be found in the following articles:
--
--  1. The X Window System
--     ACM Transactions on Graphics April 1986 Volume 5 Number 2 p. 79.
--     by Robert W. Scheifler and Jim Gettys.
--
--  2. Comparison of Windowing Systems
--     Byte Magazine November 1987 Volume 12 Number 13 p. 265.
--     by Hal L. Stern.
--
-- End of Header
----------------------------------------------------------------------------
with System ;
with X_Key_Symbols ;

package X_Types is

    X_Error      : exception; -- synchronous error returned by X server
    X_Failure    : exception; -- action dependant problem with the X server
    
    
    -- ****************************************************************
    -- * ERROR CODES
    -- ****************************************************************
    type Error_Type is
        ( Success ,     -- everything's okay
          Bad_Request , -- bad request code
          Bad_Value ,   -- int parameter out of range
          Bad_Window ,  -- parameter not a Window
          Bad_Pixmap ,  -- parameter not a Pixmap
          Bad_Atom ,    -- parameter not an Atom
          Bad_Cursor ,  -- parameter not a Cursor
          Bad_Font ,    -- parameter not a Font
          Bad_Match ,   -- parameter mismatch
          Bad_Drawable ,-- parameter not a Pixmap or Window
          Bad_Access ,  -- depending on context:
          -- key/button already grabbed
          -- attempt to free an illegal color map entry
          -- attempt to store into a read-only color map entry.
          -- attempt to modify the access control list from other
          -- than the local host.

          Bad_Alloc ,     -- insufficient resources
          Bad_Colormap ,  -- no such colormap
          Bad_GC ,        -- parameter not a GC
          Bad_ID_Choice , -- choice not in range or already used
          Bad_Name ,      -- font or color name doesn't exist
          Bad_Length ,    -- Request length incorrect
          Bad_Implementation,   -- server is defective
          Extension_Error ) ;

    -- Useful subtypes of standard type Natural
    -- limiting the size of unconstrained records on some compilers
    subtype X_Natural is Natural range 0 .. 65_535 ;
    subtype X_Positive is Natural range 1 .. 65_535 ;
    
    subtype X_Short_Natural is X_Natural range 0 .. 255 ;
    subtype Short_Natural is Natural range 0 .. 255 ;
    subtype Thin_Natural is Natural range 0 .. 6 ;

    
    package Source_Of_Resources is
        type Resource is private ;
        Null_Resource    : constant Resource ;
        Special_Resource : constant Resource ;
    private
        type Resource is  range 0 .. (2 ** 29) - 1;
        Null_Resource    : constant Resource := 0 ;
        Special_Resource : constant Resource := 1 ;
    end Source_Of_Resources ;
    
    -- Private types
    subtype Resource is Source_Of_Resources.Resource ;
    Null_Resource    : constant Resource := Source_Of_Resources.Null_Resource;
    Special_Resource : constant Resource := Source_Of_Resources.Special_Resource;

    type Atom is private;

    type Color_Map is new Resource;

    type Cursor is new Resource;

    type Display is private;

    type Drawable is new Resource;
    
    type Font is new Resource;

    type Image is private;

    type Graphic_Context is private;

    subtype Pixmap is Drawable;

    type Region is private;

    type Resource_Gc is new Resource;

    type Screen is private;

    type Visual is private;

    subtype Window is Drawable;

    type X_Id is new Resource;

    type Atom_Array is array (X_Natural range <>) of Atom;
    type Atom_List ( Length : X_Natural := 0 ) is
    record
        Atoms : Atom_Array ( 1 .. Length ) ;
    end record ;

    type Window_Array is array (X_Natural range <>) of Window;
    type Window_List ( Length : X_Natural := 0 ) is
    record
        Windows : Window_Array ( 1 .. Length ) ;
    end record ;

    -- Private constants

    Copy_Colormap_From_Parent : constant Color_Map :=
        Color_Map (Null_Resource);

    Copy_Drawable_From_Parent : constant Drawable :=
        Drawable (Null_Resource);

    Copy_Visual_From_Parent   : constant Visual;

    Image_Context             : constant Image;

    Input_Focus_Window        : constant Window :=
        Window (Special_Resource);

    None                      : constant Drawable := Drawable (Null_Resource);

    Null_Atom                 : constant Atom;
    Null_Color_Map            : constant Color_Map :=
        Color_Map (Null_Resource);
    Null_Cursor               : constant Cursor := Cursor (Null_Resource);
    Null_Display              : constant Display;
    Null_Drawable             : constant Drawable := Drawable (Null_Resource);
    Null_Font                 : constant Font := Font (Null_Resource);
    Null_Graphic_Context      : constant Graphic_Context;
    Null_Pixmap               : constant Pixmap := Null_Drawable;
    Null_Region               : constant Region;
    Null_Visual               : constant Visual;
    Null_Window               : constant Window := Null_Drawable;
    Null_X_Id                 : constant X_Id := X_Id (Null_Resource);

    Parent_Relative           : constant Pixmap := Pixmap (Special_Resource);

    Pointer_Root_Window       : constant Window := Window (Special_Resource);
    Pointer_Window            : constant Window := Null_Window;

    --
    -- Selections Predefined Properties
    --
    Xa_Primary                : constant Atom;
    Xa_Secondary              : constant Atom;
    
    
    --
    -- Predefined Data Types Properties
    --
    Xa_Arc                    : constant Atom;
    Xa_Atom                   : constant Atom;
    Xa_Bitmap                 : constant Atom;
    Xa_Cardinal               : constant Atom;
    Xa_Colormap               : constant Atom;
    Xa_Cursor                 : constant Atom;
    Xa_Drawable               : constant Atom;
    Xa_Font                   : constant Atom;
    Xa_Integer                : constant Atom;
    Xa_Pixmap                 : constant Atom;
    Xa_Point                  : constant Atom;
    Xa_Rectangle              : constant Atom;
    Xa_Rgb_Color_Map          : constant Atom;
    Xa_String                 : constant Atom;
    Xa_Visual_Id              : constant Atom;
    Xa_Window                 : constant Atom;
    Xa_Wm_Hints               : constant Atom;
    Xa_Wm_Size_Hints          : constant Atom;

    --
    -- Predefined Color Maps Standard Properties
    --
    Xa_Rgb_Best_Map           : constant Atom;
    Xa_Rgb_Blue_Map           : constant Atom;
    Xa_Rgb_Default_Map        : constant Atom;
    Xa_Rgb_Gray_Map           : constant Atom;
    Xa_Rgb_Green_Map          : constant Atom;
    Xa_Rgb_Red_Map            : constant Atom;
    
    --
    -- Cut Buffers Predefined Properties
    --
    Xa_Cut_Buffer_0           : constant Atom;
    Xa_Cut_Buffer_1           : constant Atom;
    Xa_Cut_Buffer_2           : constant Atom;
    Xa_Cut_Buffer_3           : constant Atom;
    Xa_Cut_Buffer_4           : constant Atom;
    Xa_Cut_Buffer_5           : constant Atom;
    Xa_Cut_Buffer_6           : constant Atom;
    Xa_Cut_Buffer_7           : constant Atom;
    
    --
    -- Window Manager Predefined Standard Properties
    --
    Xa_Wm_Command             : constant Atom;
    Xa_Wm_Client_Machine      : constant Atom;
    Xa_Wm_Icon_Name           : constant Atom;
    Xa_Wm_Icon_Size           : constant Atom;
    Xa_Wm_Name                : constant Atom;
    Xa_Wm_Normal_Hints        : constant Atom;
    Xa_Wm_Zoom_Hints          : constant Atom;
    Xa_Wm_Class               : constant Atom;
    Xa_Wm_Transient_For       : constant Atom;

    --
    -- Predefined Standard Font Properties
    --
    Xa_Min_Space              : constant Atom;
    Xa_Norm_Space             : constant Atom;
    Xa_Max_Space              : constant Atom;
    Xa_End_Space              : constant Atom;
    Xa_Superscript_X          : constant Atom;
    Xa_Superscript_Y          : constant Atom;
    Xa_Subscript_X            : constant Atom;
    Xa_Subscript_Y            : constant Atom;
    Xa_Underline_Position     : constant Atom;
    Xa_Underline_Thickness    : constant Atom;
    Xa_Strikeout_Ascent       : constant Atom;
    Xa_Strikeout_Descent      : constant Atom;
    Xa_Italic_Angle           : constant Atom;
    Xa_X_Height               : constant Atom;
    Xa_Quad_Width             : constant Atom;
    Xa_Weight                 : constant Atom;
    Xa_Point_Size             : constant Atom;
    Xa_Resolution             : constant Atom;
    Xa_Copyright              : constant Atom;
    Xa_Notice                 : constant Atom;
    Xa_Font_Name              : constant Atom;
    Xa_Family_Name            : constant Atom;
    Xa_Full_Name              : constant Atom;
    Xa_Cap_Height             : constant Atom;
    
    --
    -- Resource Database Predefined Properties
    --
    
    Xa_Resource_Manager       : constant Atom;
    
    --
    -- Other Predefined Properties
    --
    Xa_Last_Predefined        : constant Atom;


    -- the following are data types used throughout the X Window package
    --

    X_32_Bits : constant := 32 / System.Storage_Unit ;
    
    type Boolean_Array is array (Short_Natural range <>) of Boolean;
    pragma Pack (Boolean_Array) ;

    type Byte is  range 0 .. 255;
    for  Byte'size use 8 ;
    type Byte_Array is array (X_Natural range <>) of Byte;

    type Image_Range is range 0 .. System.Max_Int ;
    type Image_Bytes is array (Image_Range range <>) of Byte ;
    type Image_Data is access Image_Bytes ;
    
    type Map_Array is array (X_Natural range <>) of Byte;

    type String_Record ( Length : Short_Natural := 0 ) is
    record
        Chars : String ( 1 .. Length ) ;
    end record ;
    
    type String_Array is array (Short_Natural range <>) of String_Record;
    type String_List (Count : Short_Natural) is
    record
        Strings : String_Array (1..Count);
    end record ;

    type Word is  range 0 .. 65_535;
    for  Word'size use 16 ;
    type Word_Array is array (X_Natural range <>) of Word;
    type Words is access Word_Array;

    type X_Character is  range 0 .. 255;
    for  X_Character'size use 8 ;
    
    --
    -- Font Cursors
    --
    subtype Cursor_Shape is X_Character;

    X_Cursor            : constant Cursor_Shape := 0;
    Arrow               : constant Cursor_Shape := 2;
    Based_Arrow_Down    : constant Cursor_Shape := 4;
    Based_Arrow_Up      : constant Cursor_Shape := 6;
    Boat                : constant Cursor_Shape := 8;
    Bogosity            : constant Cursor_Shape := 10;
    Bottom_Left_Corner  : constant Cursor_Shape := 12;
    Bottom_Right_Corner : constant Cursor_Shape := 14;
    Bottom_Side         : constant Cursor_Shape := 16;
    Bottom_Tee          : constant Cursor_Shape := 18;
    Box_Spiral          : constant Cursor_Shape := 20;
    Center_Ptr          : constant Cursor_Shape := 22;
    Circle              : constant Cursor_Shape := 24;
    Clock               : constant Cursor_Shape := 26;
    Coffee_Mug          : constant Cursor_Shape := 28;
    Cross               : constant Cursor_Shape := 30;
    Cross_Reverse       : constant Cursor_Shape := 32;
    Crosshair           : constant Cursor_Shape := 34;
    Diamond_Cross       : constant Cursor_Shape := 36;
    Dot                 : constant Cursor_Shape := 38;
    Dot_Box_Mask        : constant Cursor_Shape := 40;
    Double_Arrow        : constant Cursor_Shape := 42;
    Draft_Large         : constant Cursor_Shape := 44;
    Draft_Small         : constant Cursor_Shape := 46;
    Draped_Box          : constant Cursor_Shape := 48;
    Exchange            : constant Cursor_Shape := 50;
    Fleur               : constant Cursor_Shape := 52;
    Gobbler             : constant Cursor_Shape := 54;
    Gumby               : constant Cursor_Shape := 56;
    Hand                : constant Cursor_Shape := 58;
    Handl_Mask          : constant Cursor_Shape := 60;
    Heart               : constant Cursor_Shape := 62;
    Icon                : constant Cursor_Shape := 64;
    Iron_Cross          : constant Cursor_Shape := 66;
    Left_Ptr            : constant Cursor_Shape := 68;
    Left_Side           : constant Cursor_Shape := 70;
    Left_Tee            : constant Cursor_Shape := 72;
    Leftbutton          : constant Cursor_Shape := 74;
    Ll_Angle            : constant Cursor_Shape := 76;
    Lr_Angle            : constant Cursor_Shape := 78;
    Man                 : constant Cursor_Shape := 80;
    Middlebutton        : constant Cursor_Shape := 82;
    Mouse               : constant Cursor_Shape := 84;
    Pencil              : constant Cursor_Shape := 86;
    Pirate              : constant Cursor_Shape := 88;
    Plus                : constant Cursor_Shape := 90;
    Question_Arrow      : constant Cursor_Shape := 92;
    Right_Ptr           : constant Cursor_Shape := 94;
    Right_Side          : constant Cursor_Shape := 96;
    Right_Tee           : constant Cursor_Shape := 98;
    Rightbutton         : constant Cursor_Shape := 100;
    Rtl_Logo            : constant Cursor_Shape := 102;
    Sailboat            : constant Cursor_Shape := 104;
    Sb_Down_Arrow       : constant Cursor_Shape := 106;
    Sb_H_Double_Arrow   : constant Cursor_Shape := 108;
    Sb_Left_Arrow       : constant Cursor_Shape := 110;
    Sb_Right_Arrow      : constant Cursor_Shape := 112;
    Sb_Up_Arrow         : constant Cursor_Shape := 114;
    Sb_V_Double_Arrow   : constant Cursor_Shape := 116;
    Shuttle             : constant Cursor_Shape := 118;
    Sizing              : constant Cursor_Shape := 120;
    Spider              : constant Cursor_Shape := 122;
    Spraycan            : constant Cursor_Shape := 124;
    Star                : constant Cursor_Shape := 126;
    Target              : constant Cursor_Shape := 128;
    Tcross              : constant Cursor_Shape := 130;
    Top_Left_Arrow      : constant Cursor_Shape := 132;
    Top_Left_Corner     : constant Cursor_Shape := 134;
    Top_Right_Corner    : constant Cursor_Shape := 136;
    Top_Side            : constant Cursor_Shape := 138;
    Top_Tee             : constant Cursor_Shape := 140;
    Trek                : constant Cursor_Shape := 142;
    Ul_Angle            : constant Cursor_Shape := 144;
    Umbrella            : constant Cursor_Shape := 146;
    Ur_Angle            : constant Cursor_Shape := 148;
    Watch               : constant Cursor_Shape := 150;
    Xterm               : constant Cursor_Shape := 152;
    Num_Glyphs          : constant Cursor_Shape := 154;

    type String_8 is array (Positive range <>) of X_Character;
    type String_Record_8 ( Length : Short_Natural := 0 ) is
    record
        Chars : String_8 ( 1 .. Length ) ;
    end record ;
    
    type Character_2b is    -- normal 16 bit characters are two bytes
    record
        Ls_Char : X_Character;
        Ms_Char : X_Character;
    end record;
    
    for Character_2b use
    record
        Ls_Char at 0 * X_32_Bits range  0 ..  7 ;
        Ms_Char at 0 * X_32_Bits range  8 .. 15 ;
    end record ;
    for Character_2b'size use 16 ;
    
    type String_16 is array (Short_Natural range <>) of Character_2b;
    type String_Record_16 ( Length : Short_Natural := 0 ) is
    record
        Chars : String_16 ( 1 .. Length ) ;
    end record ;

    type X_Long_Integer is  range - 2 ** 31 .. (2 ** 31) - 1;
    type Long_Array is array (X_Natural range <>) of X_Long_Integer;

    type Angle is delta 2.0 ** (-6)  range 0.0 .. 360.0 - 2.0 ** (-6) ;
    for Angle'Small use 2.0 ** (-6) ;
    
    type Aspect_Operand is range 1 .. (2 ** 31) - 1 ;
    
    type Bell_Volume_Type is  range - 100 .. 100;

    subtype Buffer_Range is X_Natural range 0 .. 7 ;
    
    type Dash is  range 0 .. 255;
    for  Dash'size use 8 ;

    type Dash_Array is array (Short_Natural range <>) of Dash ;
    type Dash_List ( Length : Short_Natural := 0 ) is
    record
        Mask : Dash_Array (1 .. Length) ;
    end record ;
    
    type Depth_Type is  range 0 .. (2 ** 8) - 1;
    for  Depth_Type'size use 8 ;

    type Error_Code_Type is  range 0 .. (2 ** 8 - 1);
    
    type Event_Code_Type is  range 0 .. (2 ** 7 - 1);
    
    type Hertz_Type is  range - 1 .. 1000;

    type Keycode is range 0 .. 255;
    for  Keycode'size use 8 ;
    subtype Invalid_Keycodes is Keycode range 1 .. 7 ;
    subtype Valid_Keycodes is Keycode range 8 .. Keycode'Last ;
    No_Keycode : constant Keycode := 0 ;

    type Led_Type is range 1 .. 32;

    type Major_Code_Type is  range 0 .. (2 ** 8 - 1);
    for  Major_Code_Type'size use 8 ;

    type Millimeters is  range - 2 ** 15 .. (2 ** 15) - 1;
    
    type Milliseconds_Type is  range - 1 .. 5_000;

    type Minor_Code_Type is  range 0 .. (2 ** 8 - 1);
    
    type Percent_Type is  range - 1 .. 100;
    

    type Pixel_Value is range - 2 ** 31 .. (2 ** 31) - 1;

    type Pixels is range - 2 ** 15 .. (2 ** 15) - 1;
    subtype Coordinate is Pixels;

    type Pointer_Motion is range -1 .. (2 ** 15) - 1 ;
    
    type Rgb_Value_Type is  range 0 .. 65_535;

    Full_Color : constant Rgb_Value_Type := Rgb_Value_Type'last;
    Half_Color : constant Rgb_Value_Type := Rgb_Value_Type'last / 2;
    Color_Off  : constant Rgb_Value_Type := 0;

    type Screen_Number is  range 0 .. (2 ** 15) - 1;

    type Time is  range - 2 ** 31 .. (2 ** 31) - 1;
    
    Current_Time : constant Time := 0 ;

    subtype Configure_Request_Mask_Type is Boolean_Array (0 .. 8);

    subtype Cw_Mask_Type is Boolean_Array (0 .. 6);
    
    Cw_X_Bit : constant := 0 ;
    Cw_X : constant Cw_Mask_Type :=
        Cw_Mask_Type'( Cw_X_Bit => True, others => False);
    
    Cw_Y_Bit : constant := 1 ;
    Cw_Y : constant Cw_Mask_Type :=
        Cw_Mask_Type'( Cw_Y_Bit => True, others => False);
    
    Cw_Width_Bit : constant := 2 ;
    Cw_Width : constant Cw_Mask_Type :=
        Cw_Mask_Type'( Cw_Width_Bit => True, others => False);
    
    Cw_Height_Bit : constant := 3 ;
    Cw_Height : constant Cw_Mask_Type :=
        Cw_Mask_Type'( Cw_Height_Bit => True, others => False);
    
    Cw_Border_Width_Bit : constant := 4 ;
    Cw_Border_Width : constant Cw_Mask_Type :=
        Cw_Mask_Type'( Cw_Border_Width_Bit => True, others => False);
    
    Cw_Sibling_Bit : constant := 5 ;
    Cw_Sibling : constant Cw_Mask_Type :=
        Cw_Mask_Type'( Cw_Sibling_Bit => True, others => False);
    
    Cw_Stack_Mode_Bit : constant := 6 ;
    Cw_Stack_Mode : constant Cw_Mask_Type :=
        Cw_Mask_Type'( Cw_Stack_Mode_Bit => True, others => False);

    subtype Event_Mask_Type is Boolean_Array (0 .. 31);

    No_Event_Mask              : constant Event_Mask_Type :=
        Event_Mask_Type'(others => False);
    
    Key_Press_Bit              : constant := 0 ;
    Key_Press_Mask             : constant Event_Mask_Type :=
        Event_Mask_Type'(Key_Press_Bit      => True, others => False);
    
    Key_Release_Bit            : constant := 1 ;
    Key_Release_Mask           : constant Event_Mask_Type :=
        Event_Mask_Type'(Key_Release_Bit      => True, others => False);
    
    Button_Press_Bit           : constant := 2 ;
    Button_Press_Mask          : constant Event_Mask_Type :=
        Event_Mask_Type'(Button_Press_Bit      => True, others => False);
    
    Button_Release_Bit         : constant := 3 ;
    Button_Release_Mask        : constant Event_Mask_Type :=
        Event_Mask_Type'(Button_Release_Bit => True, others => False);
    
    Enter_Window_Bit           : constant := 4 ;
    Enter_Window_Mask          : constant Event_Mask_Type :=
        Event_Mask_Type'(Enter_Window_Bit      => True, others => False);
    
    Leave_Window_Bit           : constant := 5 ;
    Leave_Window_Mask          : constant Event_Mask_Type :=
        Event_Mask_Type'(Leave_Window_Bit      => True, others => False);
    
    Pointer_Motion_Bit         : constant := 6 ;
    Pointer_Motion_Mask        : constant Event_Mask_Type :=
        Event_Mask_Type'(Pointer_Motion_Bit    => True, others => False);
    
    Pointer_Motion_Hint_Bit    : constant := 7 ;
    Pointer_Motion_Hint_Mask   : constant Event_Mask_Type :=
        Event_Mask_Type'(Pointer_Motion_Hint_Bit => True,others => False);
    
    Button_1_Motion_Bit        : constant := 8 ;
    Button_1_Motion_Mask       : constant Event_Mask_Type :=
        Event_Mask_Type'(Button_1_Motion_Bit => True, others => False);
    
    Button_2_Motion_Bit        : constant := 9 ;
    Button_2_Motion_Mask       : constant Event_Mask_Type :=
        Event_Mask_Type'(Button_2_Motion_Bit => True, others => False);
    
    Button_3_Motion_Bit        : constant := 10 ;
    Button_3_Motion_Mask       : constant Event_Mask_Type :=
        Event_Mask_Type'(Button_3_Motion_Bit => True, others => False);
    
    Button_4_Motion_Bit        : constant := 11 ;
    Button_4_Motion_Mask       : constant Event_Mask_Type :=
        Event_Mask_Type'(Button_4_Motion_Bit => True, others => False);
    
    Button_5_Motion_Bit        : constant := 12 ;
    Button_5_Motion_Mask       : constant Event_Mask_Type :=
        Event_Mask_Type'(Button_5_Motion_Bit => True, others => False);
    
    Button_Motion_Bit          : constant := 13 ;
    Button_Motion_Mask         : constant Event_Mask_Type :=
        Event_Mask_Type'(Button_Motion_Bit     => True, others => False);
    
    Keymap_State_Bit           : constant := 14 ;
    Keymap_State_Mask          : constant Event_Mask_Type :=
        Event_Mask_Type'(Keymap_State_Bit     => True, others => False);
    
    Exposure_Bit               : constant := 15 ;
    Exposure_Mask              : constant Event_Mask_Type :=
        Event_Mask_Type'(Exposure_Bit     => True, others => False);
    
    Visibility_Change_Bit      : constant := 16 ;
    Visibility_Change_Mask     : constant Event_Mask_Type :=
        Event_Mask_Type'(Visibility_Change_Bit => True, others => False);
    
    Structure_Change_Bit       : constant := 17 ;
    Structure_Change_Mask      : constant Event_Mask_Type :=
        Event_Mask_Type'(Structure_Change_Bit => True, others => False);
    
    Resize_Redirect_Bit        : constant := 18 ;
    Resize_Redirect_Mask       : constant Event_Mask_Type :=
        Event_Mask_Type'(Resize_Redirect_Bit => True, others => False);
    
    Substructure_Notify_Bit    : constant := 19 ;
    Substructure_Notify_Mask   : constant Event_Mask_Type :=
        Event_Mask_Type'(Substructure_Notify_Bit => True,others => False);
    
    Substructure_Redirect_Bit  : constant := 20 ;
    Substructure_Redirect_Mask : constant Event_Mask_Type :=
        Event_Mask_Type'(Substructure_Redirect_Bit => True,others=>False);
    
    Focus_Change_Bit           : constant := 21 ;
    Focus_Change_Mask          : constant Event_Mask_Type :=
        Event_Mask_Type'(Focus_Change_Bit     => True, others => False);
    
    Property_Change_Bit        : constant := 22 ;
    Property_Change_Mask       : constant Event_Mask_Type :=
        Event_Mask_Type'(Property_Change_Bit => True, others => False);
    
    Colormap_Change_Bit        : constant := 23 ;
    Colormap_Change_Mask       : constant Event_Mask_Type :=
        Event_Mask_Type'(Colormap_Change_Bit => True, others => False);
    
    Owner_Grab_Button_Bit      : constant := 24 ;
    Owner_Grab_Button_Mask     : constant Event_Mask_Type :=
        Event_Mask_Type'(Owner_Grab_Button_Bit => True, others => False);
    
    subtype Gc_Mask_Type is Boolean_Array (0 .. 22);

    Gc_Function_Bit : constant := 0 ;
    Gc_Function : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Function_Bit => True, others => False);
    
    Gc_Plane_Mask_Bit : constant := 1 ;
    Gc_Plane_Mask : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Plane_Mask_Bit => True, others => False);
    
    Gc_Foreground_Bit : constant := 2 ;
    Gc_Foreground : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Foreground_Bit => True, others => False);
    
    Gc_Background_Bit : constant := 3 ;
    Gc_Background : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Background_Bit => True, others => False);
    
    Gc_Line_Width_Bit : constant := 4 ;
    Gc_Line_Width : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Line_Width_Bit => True, others => False);
    
    Gc_Line_Style_Bit : constant := 5 ;
    Gc_Line_Style : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Line_Style_Bit => True, others => False);
    
    Gc_Cap_Style_Bit : constant := 6 ;
    Gc_Cap_Style : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Cap_Style_Bit => True, others => False);
    
    Gc_Join_Style_Bit : constant := 7 ;
    Gc_Join_Style : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Join_Style_Bit => True, others => False);
    
    Gc_Fill_Style_Bit : constant := 8 ;
    Gc_Fill_Style : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Fill_Style_Bit => True, others => False);
    
    Gc_Fill_Rule_Bit : constant := 9 ;
    Gc_Fill_Rule : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Fill_Rule_Bit => True, others => False);
    
    Gc_Tile_Bit : constant := 10 ;
    Gc_Tile : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Tile_Bit=> True, others => False);
    
    Gc_Stipple_Bit : constant := 11 ;
    Gc_Stipple : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Stipple_Bit=> True, others => False);
    
    Gc_Ts_X_Origin_Bit : constant := 12 ;
    Gc_Ts_X_Origin       : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Ts_X_Origin_Bit=> True, others => False);
    
    Gc_Ts_Y_Origin_Bit : constant := 13 ;
    Gc_Ts_Y_Origin       : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Ts_Y_Origin_Bit=> True, others => False);
    
    Gc_Font_Bit : constant := 14 ;
    Gc_Font : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Font_Bit=> True, others => False);
    
    Gc_Subwindow_Mode_Bit : constant := 15 ;
    Gc_Subwindow_Mode    : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Subwindow_Mode_Bit=> True, others => False);
    
    Gc_Graphics_Exposure_Bit : constant := 16 ;
    Gc_Graphics_Exposure : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Graphics_Exposure_Bit=> True, others => False);
    
    Gc_Clip_X_Origin_Bit : constant := 17 ;
    Gc_Clip_X_Origin     : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Clip_X_Origin_Bit=> True, others => False);
    
    Gc_Clip_Y_Origin_Bit : constant := 18 ;
    Gc_Clip_Y_Origin     : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Clip_Y_Origin_Bit=> True, others => False);
    
    Gc_Clip_Mask_Bit : constant := 19 ;
    Gc_Clip_Mask : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Clip_Mask_Bit=> True, others => False);
    
    Gc_Dash_Offset_Bit : constant := 20 ;
    Gc_Dash_Offset       : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Dash_Offset_Bit=> True, others => False);
    
    Gc_Dash_List_Bit : constant := 21 ;
    Gc_Dash_List : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Dash_List_Bit=> True, others => False);
    
    Gc_Arc_Mode_Bit : constant := 22 ;
    Gc_Arc_Mode : constant Gc_Mask_Type :=
        Gc_Mask_Type'( Gc_Arc_Mode_Bit=> True, others => False);

    subtype Geometry_Mask_Type is Boolean_Array (0 .. 6);

    X_Value_Bit : constant := 0 ;
    X_Value                   : constant Geometry_Mask_Type :=
        Geometry_Mask_Type'(X_Value_Bit      => True, others => False);
    
    Y_Value_Bit : constant := 1 ;
    Y_Value                   : constant Geometry_Mask_Type :=
        Geometry_Mask_Type'(Y_Value_Bit      => True, others => False);
    
    Width_Value_Bit : constant := 2 ;
    Width_Value               : constant Geometry_Mask_Type :=
        Geometry_Mask_Type'(Width_Value_Bit      => True, others => False);
    
    Height_Value_Bit : constant := 3 ;
    Height_Value              : constant Geometry_Mask_Type :=
        Geometry_Mask_Type'(Height_Value_Bit      => True, others => False);
    
    All_Values_Bit : constant := 4 ;
    All_Values                : constant Geometry_Mask_Type :=
        Geometry_Mask_Type'(All_Values_Bit      => True, others => False);
    
    X_Negative_Bit : constant := 5 ;
    X_Negative                : constant Geometry_Mask_Type :=
        Geometry_Mask_Type'(X_Negative_Bit      => True, others => False);
    
    Y_Negative_Bit : constant := 6 ;
    Y_Negative                : constant Geometry_Mask_Type :=
        Geometry_Mask_Type'(Y_Negative_Bit      => True, others => False);

    subtype Key_And_Button_Mask is Boolean_Array (0 .. 15);

    Shift_Bit                  : constant := 0 ;
    Shift_Mask                 : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Shift_Bit   => True, others => False);
    
    Lock_Bit                   : constant := 1 ;
    Lock_Mask                  : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Lock_Bit     => True, others => False);
    
    Control_Bit                : constant := 2 ;
    Control_Mask               : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Control_Bit  => True, others => False);
    
    Mod_1_Bit                  : constant := 3 ;
    Mod_1_Mask                 : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Mod_1_Bit    => True, others => False);
    
    Mod_2_Bit                  : constant := 4 ;
    Mod_2_Mask                 : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Mod_2_Bit    => True, others => False);
    
    Mod_3_Bit                  : constant := 5 ;
    Mod_3_Mask                 : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Mod_3_Bit    => True, others => False);
    
    Mod_4_Bit                  : constant := 6 ;
    Mod_4_Mask                 : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Mod_4_Bit    => True, others => False);
    
    Mod_5_Bit                  : constant := 7 ;
    Mod_5_Mask                 : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Mod_5_Bit    => True, others => False);

    Button_1_Bit               : constant := 8 ;
    Button_1_Mask              : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Button_1_Bit => True, others => False);
    
    Button_2_Bit               : constant := 9 ;
    Button_2_Mask              : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Button_2_Bit => True, others => False);
    
    Button_3_Bit               : constant := 10 ;
    Button_3_Mask              : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Button_3_Bit => True, others => False);
    
    Button_4_Bit               : constant := 11 ;
    Button_4_Mask              : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Button_4_Bit => True, others => False);
    
    Button_5_Bit               : constant := 12 ;
    Button_5_Mask              : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Button_5_Bit => True, others => False);

    All_Modifier_Mask          : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(0 .. 7 => True, others => False);
    
    Any_Modifier_Bit           : constant := 15 ;
    Any_Modifier_Mask          : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(Any_Modifier_Bit => True, others => False);
    
    Any_Button_Mask            : constant Key_And_Button_Mask :=
        Key_And_Button_Mask'(others => False);

    subtype Keyboard_Control_Mask is Boolean_Array (0 .. 7);

    Key_Click_Percent_Bit  : constant := 0 ;
    Key_Click_Percent_Mask : constant Keyboard_Control_Mask :=
        Keyboard_Control_Mask'(Key_Click_Percent_Bit => True,
                               others                => False);
    Bell_Volume_Bit        : constant := 1 ;
    Bell_Volume_Mask       : constant Keyboard_Control_Mask :=
        Keyboard_Control_Mask'(Bell_Volume_Bit       => True,
                               others                => False);
    Bell_Pitch_Bit         : constant := 2 ;
    Bell_Pitch_Mask        : constant Keyboard_Control_Mask :=
        Keyboard_Control_Mask'(Bell_Pitch_Bit        => True,
                               others                => False);
    Bell_Duration_Bit      : constant := 3 ;
    Bell_Duration_Mask     : constant Keyboard_Control_Mask :=
        Keyboard_Control_Mask'(Bell_Duration_Bit     => True,
                               others                => False);
    Led_Bit                : constant := 4 ;
    Led_Mask               : constant Keyboard_Control_Mask :=
        Keyboard_Control_Mask'(Led_Bit               => True,
                               others                => False);
    Led_Mode_Bit           : constant := 5 ;
    Led_Mode_Mask          : constant Keyboard_Control_Mask :=
        Keyboard_Control_Mask'(Led_Mode_Bit          => True,
                               others                => False);
    Keycode_Bit            : constant := 6 ;
    Keycode_Mask           : constant Keyboard_Control_Mask :=
        Keyboard_Control_Mask'(Keycode_Bit           => True,
                               others                => False);
    Auto_Repeat_Mode_Bit   : constant := 7 ;
    Auto_Repeat_Mode_Mask  : constant Keyboard_Control_Mask :=
        Keyboard_Control_Mask'(Auto_Repeat_Mode_Bit  => True,
                               others                => False);

    subtype Led_Mask_Type is Boolean_Array (0 .. 31);

    subtype Plane_Mask is Boolean_Array (0 .. 23);
    type Plane_Mask_Array is array (Short_Natural range <>) of Plane_Mask;
    All_Planes : constant Plane_Mask := (0 .. 23 => False);

    subtype Size_Hint_Mask_Type is Boolean_Array (0 .. 9);

    
    User_Specified_Position_Bit : constant := 0 ;
    User_Specified_Position     : constant Size_Hint_Mask_Type :=
        Size_Hint_Mask_Type'( User_Specified_Position_Bit => True,
                              others => False);
    
    User_Specified_Size_Bit : constant := 1 ;
    User_Specified_Size     : constant Size_Hint_Mask_Type :=
        Size_Hint_Mask_Type'( User_Specified_Size_Bit => True,
                              others => False);
    
    Program_Specified_Position_Bit : constant := 2 ;
    Program_Specified_Position     : constant Size_Hint_Mask_Type :=
        Size_Hint_Mask_Type'( Program_Specified_Position_Bit => True,
                              others => False);
    
    Program_Specified_Size_Bit : constant := 3 ;
    Program_Specified_Size     : constant Size_Hint_Mask_Type :=
        Size_Hint_Mask_Type'( Program_Specified_Size_Bit => True,
                              others => False);
    
    Program_Specified_Min_Size_Bit : constant := 4 ;
    Program_Specified_Min_Size     : constant Size_Hint_Mask_Type :=
        Size_Hint_Mask_Type'( Program_Specified_Min_Size_Bit => True,
                              others => False);
    
    Program_Specified_Max_Size_Bit : constant := 5 ;
    Program_Specified_Max_Size     : constant Size_Hint_Mask_Type :=
        Size_Hint_Mask_Type'( Program_Specified_Max_Size_Bit => True,
                              others => False);
    
    Program_Specified_Resize_Inc_Bit : constant := 6 ;
    Program_Specified_Resize_Inc     : constant Size_Hint_Mask_Type :=
        Size_Hint_Mask_Type'( Program_Specified_Resize_Inc_Bit  => True,
                              others =>  False);
    
    Program_Specified_Aspect_Bit : constant := 7 ;
    Program_Specified_Aspect     : constant Size_Hint_Mask_Type :=
        Size_Hint_Mask_Type'( Program_Specified_Aspect_Bit => True,
                              others                       => False);
    
    Program_Specified_Base_Size_Bit : constant := 8 ;
    Program_Specified_Base_Size     : constant Size_Hint_Mask_Type :=
        Size_Hint_Mask_Type'( Program_Specified_Base_Size_Bit => True,
                              others                          => False);
    
    Program_Specified_Window_Gravity_Bit : constant := 9 ;
    Program_Specified_Window_Gravity     : constant Size_Hint_Mask_Type :=
        Size_Hint_Mask_Type'( Program_Specified_Window_Gravity_Bit => True,
                              others                               => False);
    
    Program_Specified_All_Hints  : constant Size_Hint_Mask_Type :=
        Size_Hint_Mask_Type'( 2 .. 7 => True, others => False ) ;

    subtype Visual_Mask_Type is Boolean_Array (0 .. 9);
    
    Visual_No_Bit : constant  := 0 ;
    Visual_No_Mask            : constant Visual_Mask_Type :=
        Visual_Mask_Type'(Visual_No_Bit      => True, others => False);
    
    Visual_Id_Bit : constant  := 1 ;
    Visual_Id_Mask            : constant Visual_Mask_Type :=
        Visual_Mask_Type'(Visual_Id_Bit      => True, others => False);
    
    Visual_Screen_Bit : constant  := 2 ;
    Visual_Screen_Mask        : constant Visual_Mask_Type :=
        Visual_Mask_Type'(Visual_Screen_Bit      => True, others => False);
    
    Visual_Depth_Bit : constant  := 3 ;
    Visual_Depth_Mask         : constant Visual_Mask_Type :=
        Visual_Mask_Type'(Visual_Depth_Bit      => True, others => False);
    
    Visual_Class_Bit : constant  := 4 ;
    Visual_Class_Mask         : constant Visual_Mask_Type :=
        Visual_Mask_Type'(Visual_Class_Bit      => True, others => False);
    
    Visual_Red_Bit : constant  := 5 ;
    Visual_Red_Mask           : constant Visual_Mask_Type :=
        Visual_Mask_Type'(Visual_Red_Bit      => True, others => False);
    
    Visual_Green_Bit : constant  := 6 ;
    Visual_Green_Mask         : constant Visual_Mask_Type :=
        Visual_Mask_Type'(Visual_Green_Bit      => True, others => False);
    
    Visual_Blue_Bit : constant  := 7 ;
    Visual_Blue_Mask          : constant Visual_Mask_Type :=
        Visual_Mask_Type'(Visual_Blue_Bit      => True, others => False);
    
    Visual_Colormap_Size_Bit : constant  := 8 ;
    Visual_Colormap_Size_Mask : constant Visual_Mask_Type :=
        Visual_Mask_Type'(Visual_Colormap_Size_Bit => True, others => False);
    
    Visual_Bits_Per_Rgb_Bit : constant  := 9 ;
    Visual_Bits_Per_Rgb_Mask  : constant Visual_Mask_Type :=
        Visual_Mask_Type'(Visual_Bits_Per_Rgb_Bit => True, others => False);
    
    Visual_All_Mask           : constant Visual_Mask_Type :=
        Visual_Mask_Type'(others => True);

    subtype Wa_Mask_Type is Boolean_Array (0 .. 14);
    
    Wa_Background_Pixmap_Bit : constant := 0 ;
    Wa_Background_Pixmap : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Background_Pixmap_Bit      => True, others => False);
    
    Wa_Background_Pixel_Bit : constant := 1 ;
    Wa_Background_Pixel  : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Background_Pixel_Bit      => True, others => False);
    
    Wa_Border_Pixmap_Bit : constant := 2 ;
    Wa_Border_Pixmap : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Border_Pixmap_Bit      => True, others => False);
    
    Wa_Border_Pixel_Bit : constant := 3 ;
    Wa_Border_Pixel : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Border_Pixel_Bit      => True, others => False);
    
    Wa_Bit_Gravity_Bit : constant := 4 ;
    Wa_Bit_Gravity : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Bit_Gravity_Bit      => True, others => False);
    
    Wa_Win_Gravity_Bit : constant := 5 ;
    Wa_Win_Gravity : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Win_Gravity_Bit      => True, others => False);
    
    Wa_Backing_Store_Bit : constant := 6 ;
    Wa_Backing_Store : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Backing_Store_Bit      => True, others => False);
    
    Wa_Backing_Planes_Bit : constant := 7 ;
    Wa_Backing_Planes : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Backing_Planes_Bit      => True, others => False);
    
    Wa_Backing_Pixel_Bit : constant := 8 ;
    Wa_Backing_Pixel : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Backing_Pixel_Bit      => True, others => False);
    
    Wa_Override_Redirect_Bit : constant := 9 ;
    Wa_Override_Redirect : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Override_Redirect_Bit      => True, others => False);
    
    Wa_Save_Under_Bit : constant := 10 ;
    Wa_Save_Under : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Save_Under_Bit     => True, others => False);
    
    Wa_Event_Mask_Bit : constant := 11 ;
    Wa_Event_Mask : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Event_Mask_Bit     => True, others => False);
    
    Wa_Dont_Propagate_Bit : constant := 12 ;
    Wa_Dont_Propagate : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Dont_Propagate_Bit     => True, others => False);
    
    Wa_Colormap_Bit : constant := 13 ;
    Wa_Colormap : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Colormap_Bit     => True, others => False);
    
    Wa_Cursor_Bit : constant := 14 ;
    Wa_Cursor : constant Wa_Mask_Type :=
        Wa_Mask_Type'(Wa_Cursor_Bit     => True, others => False);

    subtype Wm_Hint_Mask_Type is Boolean_Array (0 .. 6);
    
    Input_Bit : constant  := 0 ;
    Input_Hint         : constant Wm_Hint_Mask_Type :=
        Wm_Hint_Mask_Type'(Input_Bit      => True, others => False);
    
    Initial_State_Bit : constant  := 1 ;
    Initial_State_Hint : constant Wm_Hint_Mask_Type :=
        Wm_Hint_Mask_Type'(Initial_State_Bit      => True, others => False);
    
    Icon_Pixmap_Bit : constant  := 2 ;
    Icon_Pixmap_Hint   : constant Wm_Hint_Mask_Type :=
        Wm_Hint_Mask_Type'(Icon_Pixmap_Bit      => True, others => False);
    
    Icon_Window_Bit : constant  := 3 ;
    Icon_Window_Hint   : constant Wm_Hint_Mask_Type :=
        Wm_Hint_Mask_Type'(Icon_Window_Bit      => True, others => False);
    
    Icon_Position_Bit : constant  := 4 ;
    Icon_Position_Hint : constant Wm_Hint_Mask_Type :=
        Wm_Hint_Mask_Type'(Icon_Position_Bit      => True, others => False);
    
    Icon_Mask_Bit : constant  := 5 ;
    Icon_Mask_Hint     : constant Wm_Hint_Mask_Type :=
        Wm_Hint_Mask_Type'(Icon_Mask_Bit      => True, others => False);
    
    Window_Group_Bit : constant  := 6 ;
    Window_Group_Hint  : constant Wm_Hint_Mask_Type :=
        Wm_Hint_Mask_Type'(Window_Group_Bit      => True, others => False);
    
    All_Hints          : constant Wm_Hint_Mask_Type :=
        Wm_Hint_Mask_Type'( 0 | 2 .. 5 => True, 1 | 6 => False ) ;
    --    Input_Hint or Icon_Pixmap_Hint or Icon_Window_Hint or
    --    Icon_Position_Hint or Icon_Mask_Hint


    type Key_Vector_Mask_Type is
        array (Keycode range Keycode'first .. Keycode'last) of Boolean;
    pragma Pack (Key_Vector_Mask_Type);


    type Access_Control_Mode is (Enable, Disable);

    type Allow_Event_Mode is
        ( Async_Pointer,
          Sync_Pointer,
          Replay_Pointer,
          Async_Keyboard,
          Sync_Keyboard,
          Replay_Keyboard,
          Async_Both,
          Sync_Both);

    type Arc_Mode_Type is (Arc_Chord, Arc_Pie_Slice);

    type Auto_Repeat_Mode_Type is
        ( Auto_Repeat_Off,
          Auto_Repeat_On,
          Auto_Repeat_Default);

    type Backing_Store_Type is (Not_Useful, When_Mapped, Always);

    type Bitmap_Status_Type is
        ( Bitmap_Success,
          Bitmap_Open_Failed,
          Bitmap_File_Invalid,
          Bitmap_No_Memory);

    type Blanking_Type is
        ( Dont_Prefer_Blanking,
          Prefer_Blanking,
          Default_Blanking);

    type Button_Name_Type is
        ( Any_Button,
          Button_1,
          Button_2,
          Button_3,
          Button_4,
          Button_5);

    type Cap_Style_Type is
        ( Cap_Not_Last,
          Cap_Butt,
          Cap_Round,
          Cap_Projecting);

    type Change_Mode is (Insert, Delete);

    type Circulation_Request is (Place_On_Top, Place_On_Bottom);

    type Close_Mode is (Destroy_All, Retain_Permanent, Retain_Temporary);

    type Colormap_Allocator is (Allocate_None, Allocate_All);

    type Colormap_State_Type is (Uninstalled, Installed);
    subtype Colormap_Notification is Colormap_State_Type;

    type Coordinate_Mode_Type is (Origin, Previous);

    type Direction_Type is (Raise_Lowest, Lower_Highest);
    
    type Extension_Name is
        ( Core_Protocol,  -- meaning "not an extension"
          Shapes,
          Input,
          Pex,
          Vex ) ;
    
    type Extension_Support is
        ( Not_Yet_Supported, -- XInAda does not support this extension
          Supported,         -- XInAda support this extension
          Available ) ;      -- The connected display support this extension
    
    type Event_Queue_Mode_Type is
        ( Queued_Already,
          Queued_After_Flush,
          Queued_After_Reading);

    type Event_Type is
        ( Error,
          Reply,
          Key_Press,
          Key_Release,
          Button_Press,
          Button_Release,
          Motion_Notify,
          Enter_Notify,
          Leave_Notify,
          Focus_In,
          Focus_Out,
          Keymap_Notify,
          Expose,
          Graphics_Expose,
          No_Expose,
          Visibility_Notify,
          Create_Notify,
          Destroy_Notify,
          Unmap_Notify,
          Map_Notify,
          Map_Request,
          Reparent_Notify,
          Configure_Notify,
          Configure_Request,
          Gravity_Notify,
          Resize_Request,
          Circulate_Notify,
          Circulate_Request,
          Property_Notify,
          Selection_Clear,
          Selection_Request,
          Selection_Notify,
          Colormap_Notify,
          Client_Message,
          Mapping_Notify,
          Extension_Trigger);

    type Exposure_Type is
        ( Dont_Allow_Exposures,
          Allow_Exposures,
          Default_Exposures);

    type Fill_Rule_Type is (Even_Odd_Rule, Winding_Rule);

    type Fill_Style_Type is
        ( Fill_Solid,
          Fill_Tiled,
          Fill_Stippled,
          Fill_Opaque_Stippled);

    type Font_Direction is (Left_To_Right, Right_To_Left);

    type Grab_Mode is (Sync, Async);

    type Grab_Reply_Status is
        ( Grab_Success,
          Already_Grabbed,
          Grab_Invalid_Time,
          Grab_Not_Viewable,
          Grab_Frozen);

    type Gravity_Type is
        ( Forget_Gravity,
          Northwest_Gravity,
          North_Gravity,
          Northeast_Gravity,
          West_Gravity,
          Center_Gravity,
          East_Gravity,
          Southwest_Gravity,
          South_Gravity,
          Southeast_Gravity,
          Static_Gravity);
    
    Unmap_Gravity             : constant Gravity_Type := Forget_Gravity;

    type Gx_Function_Code is
        ( Gx_Clear,
          Gx_And,
          Gx_And_Reverse,
          Gx_Copy,
          Gx_And_Inverted,
          Gx_Noop,
          Gx_Xor,
          Gx_Or,
          Gx_Nor,
          Gx_Equiv,
          Gx_Invert,
          Gx_Or_Reverse,
          Gx_Copy_Inverted,
          Gx_Or_Inverted,
          Gx_Nand,
          Gx_Set);

    type Initial_State_Type is
        ( Dont_Care_State,
          Normal_State,
          Zoom_State,
          Iconic_State,
          Inactive_State);

    type Join_Style_Type is (Join_Miter, Join_Round, Join_Bevel);

    type Led_Mode_Type is (Led_Mode_Off, Led_Mode_On);

    type Line_Style_Type is
        ( Line_Solid,
          Line_On_Off_Dash,
          Line_Double_Dash);

    type Map_State_Type is (Is_Unmapped, Is_Unviewable, Is_Viewable);

    type Mapping_Request_Type is
        ( Mapping_Modifier,
          Mapping_Keyboard,
          Mapping_Pointer);

    type Map_Status is (Success, Busy, Failed);

    type Modifier_Type is
        ( Shift,
          Lock,
          Control,
          Mod_1,
          Mod_2,
          Mod_3,
          Mod_4,
          Mod_5);
    
    type Notify_Detail is
        ( Ancestor,
          Virtual,
          Inferior,
          Nonlinear,
          Nonlinear_Virtual,
          Pointer,
          Pointer_Root,
          Detail_None);

    type Notify_Detail_Type is
        ( Notify_Ancestor,
          Notify_Virtual,
          Notify_Inferior,
          Notify_Nonlinear,
          Notify_Nonlinear_Virtual,
          Notify_Pointer,
          Notify_Pointer_Root,
          Notify_Detail_None);

    type Notify_Modes is (Normal, Grab, Ungrab, While_Grabbed);

    type Notify_Mode_Type is
        ( Notify_Normal,
          Notify_Grab,
          Notify_Ungrab,
          Notify_While_Grabbed);
    Notify_Hint                : constant Notify_Mode_Type := Notify_Grab;

    type Ordering_Type is (Unsorted, Y_Sorted, Yx_Sorted, Yx_Banded);

    type Order_Type is (Lsb_First, Msb_First);

    type Pixmap_Format is (Xy_Bitmap, Xy_Pixmap, Z_Pixmap);

    type Placement_Type is (Place_On_Top, Place_On_Bottom);

    type Property_Format_Type is (Format_8, Format_16, Format_32);
    for Property_Format_Type use ( Format_8  =>  8,
                                   Format_16 => 16,
                                   Format_32 => 32 ) ;

    type Property_Mode is (Replace, Prepend, Append);

    type Property_Notification is (New_Value, Delete);

    type Property_State_Type is (New_Value, Delete);

    type Protocol_Family is (Internet, Decnet, Chaos);

    type Request_Type is 
        ( Create_Window_Request,
          Change_Window_Attributes_Request,
          Get_Window_Attributes_Request,
          Destroy_Window_Request,
          Destroy_Subwindows_Request,
          Change_Save_Set_Request,
          Reparent_Window_Request,
          Map_Window_Request,
          Map_Subwindows_Request,
          Unmap_Window_Request,
          Unmap_Subwindows_Request,
          Configure_Window_Request,
          Circulate_Window_Request,
          Get_Geometry_Request,
          Query_Tree_Request,
          Intern_Atom_Request,
          Get_Atom_Name_Request,
          Change_Property_Request,
          Delete_Property_Request,
          Get_Property_Request,
          List_Properties_Request,
          Set_Selection_Owner_Request,
          Get_Selection_Owner_Request,
          Convert_Selection_Request,
          Send_Event_Request,
          Grab_Pointer_Request,
          Ungrab_Pointer_Request,
          Grab_Button_Request,
          Ungrab_Button_Request,
          Change_Active_Pointer_Grab_Request,
          Grab_Keyboard_Request,
          Ungrab_Keyboard_Request,
          Grab_Key_Request,
          Ungrab_Key_Request,
          Allow_Events_Request,
          Grab_Server_Request,
          Ungrab_Server_Request,
          Query_Pointer_Request,
          Get_Motion_Events_Request,
          Translate_Coordinates_Request,
          Warp_Pointer_Request,
          Set_Input_Focus_Request,
          Get_Input_Focus_Request,
          Query_Keymap_Request,
          Open_Font_Request,
          Close_Font_Request,
          Query_Font_Request,
          Query_Text_Extents_Request,
          List_Fonts_Request,
          List_Fonts_With_Info_Request, 
          Set_Font_Path_Request,
          Get_Font_Path_Request,
          Create_Pixmap_Request,
          Free_Pixmap_Request,
          Create_GC_Request,
          Change_GC_Request,
          Copy_GC_Request,
          Set_Dashes_Request,
          Set_Clip_Rectangles_Request,
          Free_GC_Request,
          Clear_Area_Request,
          Copy_Area_Request,
          Copy_Plane_Request,
          Poly_Point_Request,
          Poly_Line_Request,
          Poly_Segment_Request,
          Poly_Rectangle_Request,
          Poly_Arc_Request,
          Fill_Poly_Request,
          Poly_Fill_Rectangle_Request,
          Poly_Fill_Arc_Request,
          Put_Image_Request,
          Get_Image_Request,
          Poly_Text_8_Request,
          Poly_Text_16_Request,
          Image_Text_8_Request,
          Image_Text_16_Request,
          Create_Colormap_Request,
          Free_Colormap_Request,
          Copy_Colormap_And_Free_Request,
          Install_Colormap_Request,
          Uninstall_Colormap_Request,
          List_Installed_Colormaps_Request,
          Allocate_Color_Request,
          Allocate_Named_Color_Request,
          Allocate_Color_Cells_Request,
          Allocate_Color_Planes_Request,
          Free_Colors_Request,
          Store_Colors_Request,
          Store_Named_Color_Request,
          Query_Colors_Request,
          Lookup_Color_Request,
          Create_Cursor_Request,
          Create_Glyph_Cursor_Request,
          Free_Cursor_Request,
          Recolor_Cursor_Request,
          Query_Best_Size_Request,
          Query_Extension_Request,
          List_Extensions_Request,
          Change_Keyboard_Mapping_Request,
          Get_Keyboard_Mapping_Request,
          Change_Keyboard_Control_Request,
          Get_Keyboard_Control_Request,
          Bell_Request,
          Change_Pointer_Control_Request,
          Get_Pointer_Control_Request,
          Set_Screen_Saver_Request,
          Get_Screen_Saver_Request,
          Change_Hosts_Request,
          List_Hosts_Request,
          Set_Access_Control_Request,
          Set_Close_Down_Mode_Request,
          Kill_Client_Request,
          Rotate_Properties_Request,       
          Force_Screen_Saver_Request,      
          Set_Pointer_Mapping_Request,
          Get_Pointer_Mapping_Request,
          Set_Modifier_Mapping_Request,    
          Get_Modifier_Mapping_Request,    
          No_Operation_Request,
          Extension_Request ) ;
        
    for Request_Type use 
        (  1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
          11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
          21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
          31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
          41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
          51,  52,  53,  54,  55,  56,  57,  58,  59,  60,
          61,  62,  63,  64,  65,  66,  67,  68,  69,  70,
          71,  72,  73,  74,  75,  76,  77,  78,  79,  80,
          81,  82,  83,  84,  85,  86,  87,  88,  89,  90,
          91,  92,  93,  94,  95,  96,  97,  98,  99, 100,
         101, 102, 103, 104, 105, 106, 107, 108, 109, 110,
         111, 112, 113, 114, 115, 116, 117, 118, 119, 127, 128 ) ;
    for Request_Type'size use 8 ;
    
    type Revert_Mode is
        ( Revert_To_None,
          Revert_To_Pointer_Root,
          Revert_To_Parent);

    type Screen_Saver_Mode is (Active, Reset);

    type Shape_Class_Type is
        ( Tile_Class,
          Cursor_Class,
          Stipple_Class);

    type Shape_Type is (Complex, Convex, Nonconvex);
    
    type Stack_Mode_Type is (Above, Below, Top_If, Bottom_If, Opposite);

    type Subwindow_Mode_Type is (Clip_By_Children, Include_Inferiors);

    type Synchronize_Mode is (Off, On, Debug);

    type Visibility_Notify_Type is
        ( Unobscured,
          Partially_Obscured,
          Fully_Obscured);

    type Visibility_Type is
        ( Visibility_Unobscured,
          Visibility_Partially_Obscured,
          Visibility_Full_Obscured);

    type Visual_Class_Type is
        ( Static_Gray,
          Gray_Scale,
          Static_Color,
          Pseudo_Color,
          True_Color,
          Direct_Color);

    type Window_Class is (Copy_Class_From_Parent, Input_Output, Input_Only);
    for  Window_Class'size use 16 ;


    type Point is
    record
        X, Y : Coordinate := 0;
    end record;
    
    for Point use
    record
        X   at 0 * X_32_Bits range  0 .. 15 ;
        Y   at 0 * X_32_Bits range 16 .. 31 ;
    end record;
    
    type Point_Array is array (X_Natural range <>) of Point;

    type Segment is
    record
        P_1 : Point;
        P_2 : Point;
    end record;
    type Segment_Array is array (X_Natural range <>) of Segment;

    type Rectangle is
    record
        Origin : Point  ;
        Width  : Pixels     := 0;
        Height : Pixels     := 0;
    end record;
    
    for Rectangle use
    record
        Origin   at 0 * X_32_Bits range  0 .. 31 ;
        Width    at 1 * X_32_Bits range  0 .. 15 ;
        Height   at 1 * X_32_Bits range 16 .. 31 ;
    end record;
    
    type Rectangle_Array is array (X_Natural range <>) of Rectangle;

    type Arc is
    record
        Bounds  : Rectangle ;
        Angle_1 : Angle      := 0.0;
        Angle_2 : Angle      := 0.0;
    end record;
    
    for Arc use
    record
        Bounds      at 0 * X_32_Bits range  0 .. 63 ;
        Angle_1     at 2 * X_32_Bits range  0 .. 15 ;
        Angle_2     at 2 * X_32_Bits range 16 .. 31 ;
    end record;
    
    type Arc_Array is array (X_Natural range <>) of Arc;

    type Aspect_Ratio is
    record
        Numerator   : Aspect_Operand := 1;
        Denominator : Aspect_Operand := 1;
    end record;
    
    for Aspect_Ratio use
    record
        Numerator     at 0 * X_32_Bits range  0 .. 31 ;
        Denominator   at 1 * X_32_Bits range  0 .. 31 ;
    end record;
    
    type Icon_Size_Record is
    record
        Min_Width  : Pixels;
        Min_Height : Pixels;
        Max_Width  : Pixels;
        Max_Height : Pixels;
        Width_Inc  : Pixels;
        Height_Inc : Pixels;
    end record;
    type Icon_Size_Array is array (X_Natural range <>) of Icon_Size_Record;
    type Icon_Size_List ( Length : X_Natural := 0 ) is
    record
        Icon_Sizes : Icon_Size_Array ( 1 .. Length ) ;
    end record ;

    type Property_Data ( Format  : Property_Format_Type := Format_8 ;
                         N_Items : X_Natural      := 0 ) is
    record
        case Format is
        when Format_8   =>  Data_8  : Byte_Array (1 .. N_Items) ;
        when Format_16  =>  Data_16 : Word_Array (1 .. N_Items) ;
        when Format_32  =>  Data_32 : Long_Array (1 .. N_Items) ;
        end case;
    end record ;

    type Size_Hint_Record is
    record
        Flags       : Size_Hint_Mask_Type;
        Bounds      : Rectangle;
        Min_Width   : Pixels;
        Min_Height  : Pixels;
        Max_Width   : Pixels;
        Max_Height  : Pixels;
        Width_Inc   : Pixels;
        Height_Inc  : Pixels;
        Min_Aspect  : Aspect_Ratio;
        Max_Aspect  : Aspect_Ratio;
        Base_Width  : Pixels;
        Base_Height : Pixels ;
        Win_Gravity : Gravity_Type ;
    end record;

    type Visual_Info_Record is
    record
        About         : Visual;
        Visual_Id     : X_Id;
        Screen_Id     : Screen_Number;
        Depth         : Depth_Type;
        Class         : Visual_Class_Type;
        Red_Mask      : Plane_Mask;
        Green_Mask    : Plane_Mask;
        Blue_Mask     : Plane_Mask;
        Colormap_Size : Integer;
        Bits_Per_Rgb  : X_Natural;
    end record;
    type Visual_Info_Array is array (X_Natural range <>) of Visual_Info_Record;
    type Visual_Info_List ( Length : X_Short_Natural := 0 ) is
    record
        Visual_Infos : Visual_Info_Array ( 1 .. Length ) ;
    end record ;

    type Set_Window_Attributes_Record is
    record
        Background_Pixmap : Pixmap; -- background or None or ParentRelative
        Background_Pixel  : Pixel_Value        := 0; -- background pixel
        Border_Pixmap     : Pixmap; -- border of the window
        Border_Pixel      : Pixel_Value        := 1; -- border pixel value
        Bit_Gravity       : Gravity_Type       := Forget_Gravity;
        Window_Gravity    : Gravity_Type       := Northwest_Gravity;
        Backing_Store     : Backing_Store_Type := Not_Useful;
        Backing_Planes    : Plane_Mask         := All_Planes;
        -- planes to be preserved
        Backing_Pixel     : Pixel_Value        := 0;
        -- value to use in restoring planes
        Save_Under        : Boolean            := False;
        -- should bits under be saved?
        Event_Mask        : Event_Mask_Type :=  -- set of events that
            Event_Mask_Type'(others => False); -- should be saved
        Do_Not_Propagate  : Event_Mask_Type :=  -- set of events that
            Event_Mask_Type'(others => False);
        -- should not propagate
        Override_Redirect : Boolean            := False;
        Current_Color_Map : Color_Map;
        -- to be associated with window
        Current_Cursor    : Cursor; -- to be displayed (or None)
    end record;

    type Window_Attributes_Record is
    record
        Origin                : Point; -- location of window
        Width, Height         : Pixels; -- width and height of window
        Border_Width          : Pixels; -- border width of window
        Depth                 : Depth_Type; -- depth of window
        Visuals               : Visual; -- the associated visual structure
        Root_Id               : Window; -- root of screen containing window
        Class                 : Window_Class;
        Bit_Gravity           : Gravity_Type;
        Win_Gravity           : Gravity_Type;
        Backing_Store         : Backing_Store_Type;
        Backing_Planes        : Plane_Mask; -- planes to be preserved
        Backing_Pixel         : Pixel_Value; -- be used when restoring planes
        Save_Under            : Boolean; -- should bits under be saved?
        Colormap              : Color_Map; -- to be associated with the window
        Map_Installed         : Boolean; -- is color map currently installed
        Map_State             : Map_State_Type;
        All_Event_Masks       : Event_Mask_Type; -- set of events all people
                                                 -- have interest in
        Your_Event_Mask       : Event_Mask_Type; -- my event mask
        Do_Not_Propagate_Mask : Event_Mask_Type; -- dont propagate that events
        Override_Redirect     : Boolean;
        Screen_Id             : Screen_Number; -- num of correct screen
    end record;

    -- Data structure for Reconfigure_Window
    type Window_Changes_Record is
    record
        Bounds       : Rectangle;
        Border_Width : Pixels;
        Sibling      : Window;
        Stack_Mode   : Stack_Mode_Type;
    end record;

    type Wm_Hint_Record is
    record
        Flags         : Wm_Hint_Mask_Type;
        Input         : Boolean;
        Initial_State : Initial_State_Type;
        Icon_Pixmap   : Pixmap;
        Icon_Window   : Window;
        Icon_Position : Point;
        Icon_Mask     : Pixmap;
        Window_Group  : X_Id;
    end record;

    type Wm_Class_Hint_Record is
    record
        Res_Name  : String_Record;
        Res_Class : String_Record;
    end record;

    -- Events records

    type Error_Event_Record (Error  : Error_Type := Success ) is
    record
        Minor_Code : Minor_Code_Type; -- of failed request
        case Error is
        when Success | Bad_Request | Bad_Match | Bad_Access |
            Bad_Alloc | Bad_Name | Bad_Length | Bad_Implementation =>
                
            Bad_Request  : Request_Type; -- Name of failed request
            
        when Bad_Value =>
            Wrong_Request : Request_Type; -- Name of failed request
            Value         : X_Long_Integer ;
            
        when Bad_Window | Bad_Pixmap | Bad_Drawable | Bad_Cursor |
            Bad_Font | Bad_Colormap | Bad_Gc | Bad_Id_Choice => 
            Out_Request  : Request_Type; -- Name of failed request
            Resource_Id  : Resource;
            
        when Bad_Atom =>
            Faulty_Request : Request_Type; -- Name of failed request
            Atom_Id        : Atom ;
            
        when Extension_Error =>
            Extension    : Extension_Name ;
            Error_Code   : Error_Code_Type; -- for extension error handling
            Info         : X_Long_Integer ; -- additionnal information
            Data         : Long_Array (1 .. 5) ; -- Additionnal data
        end case ;
    end record;

    -- Definitions of specific events.

    type Key_Event_Record is
    record
        Window_Id    : Drawable;  -- "event" window it is reported relative to
        Root         : Drawable;  -- root window that the event occured on
        Subwindow    : Drawable;  -- child window
        Event_Time   : Time;    -- milliseconds
        XY           : Point; -- pointer x, y coordinates in event window
        XY_Root      : Point; -- coordinates relative to root
        State        : Key_And_Button_Mask; -- key or button mask
        Key_Code     : Keycode; -- detail
        Same_Screen  : Boolean; -- same screen flag
    end record;


    type Button_Event_Record is
    record
        Window_Id    : Drawable;  -- "event" window it is reported relative to
        Root         : Drawable;  -- root window that the event occured on
        Subwindow    : Drawable;  -- child window           
        Event_Time   : Time;    -- milliseconds             
        XY           : Point; -- pointer x, y coordinates in event window
        XY_Root      : Point; -- coordinates relative to root
        State        : Key_And_Button_Mask; -- key or button mask   
        Button       : Button_Name_Type; -- detail      
        Same_Screen  : Boolean; -- same screen flag
    end record;

    type Motion_Event_Record is
    record
        Window_Id    : Drawable;  -- "event" window it is reported relative to
        Root         : Drawable;  -- root window that the event occured on
        Subwindow    : Drawable;  -- child window           
        Event_Time   : Time;    -- milliseconds             
        XY           : Point; -- pointer x, y coordinates in event window
        XY_Root      : Point; -- coordinates relative to root
        State        : Key_And_Button_Mask; -- key or button mask   
        Is_Hint      : Notify_Mode_Type; -- detail      
        Same_Screen  : Boolean; -- same screen flag
    end record;

    type Crossing_Event_Record is
    record
        Window_Id    : Drawable;  -- "event" window it is reported relative to
        Root         : Drawable;  -- root window that the event occured on
        Subwindow    : Drawable;  -- child window           
        Event_Time   : Time;    -- milliseconds             
        XY           : Point; -- pointer x, y coordinates in event window
        XY_Root      : Point; -- coordinates relative to root
        Mode         : Notify_Mode_Type;
        Detail       : Notify_Detail_Type;
        Same_Screen  : Boolean; -- same screen flag
        Focus        : Boolean; -- boolean focus
        State        : Key_And_Button_Mask; -- key or button mask
    end record;

    type Focus_Change_Event_Record is
    record
        Window_Id : Drawable; -- window of event
        Mode      : Notify_Mode_Type;
        Detail    : Notify_Detail_Type;
    end record;

    type Keymap_Event_Record is
    record
        Window_Id  : Drawable;
        Key_Vector : Key_Vector_Mask_Type;
    end record;

    type Expose_Event_Record is
    record
        Window_Id : Drawable;
        XY        : Point;
        Width     : Pixels;
        Height    : Pixels;
        Count     : X_Natural; -- if non-zero, at least this many more
    end record;

    type Graphics_Expose_Event_Record is
    record
        Drawable_Id : Drawable;
        XY          : Point;
        Width       : Pixels;
        Height      : Pixels;
        Count       : X_Natural; -- if non-zero, at least this many more
        Major_Code  : Major_Code_Type;  -- core is CopyArea or CopyPlane
        Minor_Code  : Minor_Code_Type;  -- not defined in the core
    end record;

    type No_Expose_Event_Record is
    record
        Drawable_Id : Drawable;
        Major_Code  : Major_Code_Type;  -- core is CopyArea or CopyPlane
        Minor_Code  : Minor_Code_Type;  -- not defined in the core
    end record;

    type Visibility_Event_Record is
    record
        Window_Id : Drawable;
        State     : Visibility_Type;
    end record;

    type Create_Window_Event_Record is
    record
        Parent            : Drawable; -- parent of the window   
        Window_Id         : Drawable; -- window id of window created
        XY                : Point; -- window location
        Width, Height     : Pixels; -- size of window       
        Border_Width      : Pixels;
        Override_Redirect : Boolean; -- creation should be overridden
    end record;

    type Destroy_Window_Event_Record is
    record
        Event_Window : Drawable;
        Window_Id    : Drawable;
    end record;

    type Unmap_Event_Record is
    record
        Event_Window   : Drawable;
        Window_Id      : Drawable;
        From_Configure : Boolean;
    end record;

    type Map_Event_Record is
    record
        Event_Window      : Drawable;
        Window_Id         : Drawable;
        Override_Redirect : Boolean; -- is override set...
    end record;

    type Map_Request_Event_Record is
    record
        Parent    : Drawable;
        Window_Id : Drawable;
    end record;

    type Reparent_Event_Record is
    record
        Event_Window      : Drawable;
        Window_Id         : Drawable;
        Parent            : Drawable;
        XY                : Point;
        Override_Redirect : Boolean;
    end record;

    type Resize_Request_Event_Record is
    record
        Window_Id : Drawable;
        Width     : Pixels;
        Height    : Pixels;
    end record;

    type Gravity_Event_Record is
    record
        Event_Window : Drawable;
        Window_Id    : Drawable;
        XY           : Point;
    end record;

    type Configure_Event_Record is
    record
        Event_Window      : Drawable;
        Window_Id         : Drawable;
        XY                : Point;
        Width             : Pixels;
        Height            : Pixels;
        Border_Width      : Pixels;
        Above             : Drawable;
        Override_Redirect : Boolean;
    end record;

    type Configure_Request_Event_Record is
    record
        Parent       : Drawable;
        Window_Id    : Drawable;
        XY           : Point;
        Width        : Pixels;
        Height       : Pixels;
        Border_Width : Pixels;
        Above        : Drawable;
        Detail       : Stack_Mode_Type;
        Value_Mask   : Configure_Request_Mask_Type;
    end record;

    type Circulate_Event_Record is
    record
        Event_Window : Drawable;
        Window_Id    : Drawable;
        Place        : Placement_Type;
    end record;

    type Circulate_Request_Event_Record is
    record
        Parent    : Drawable;
        Window_Id : Drawable;
        Place     : Placement_Type;
    end record;

    type Property_Event_Record is
    record
        Window_Id  : Drawable;
        Atom_Id    : Atom;
        Time_Stamp : Time;
        State      : Property_State_Type;
    end record;

    type Selection_Clear_Event_Record is
    record
        Owner      : Drawable;
        Selection  : Atom;
        Time_Stamp : Time;
    end record;

    type Selection_Request_Event_Record is
    record
        Owner      : Drawable;
        Requestor  : Drawable;
        Selection  : Atom;
        Target     : Atom;
        Property   : Atom;
        Time_Stamp : Time;
    end record;

    type Selection_Event_Record is
    record
        Requestor  : Drawable;
        Selection  : Atom;
        Target     : Atom;
        Property   : Atom;
        Time_Stamp : Time;
    end record;

    type Colormap_Event_Record is
    record
        Window_Id : Drawable;
        Colormap  : Color_Map;
        New_Map   : Boolean;
        State     : Colormap_State_Type;
    end record;

    type Client_Message_Event_Record
         (Format : Property_Format_Type := Format_8) is
    record
        Window_Id    : Drawable;
        Message_Type : Atom ;
        case Format is
        when Format_8   =>  Data_8 : Byte_Array (1 .. 20);
        when Format_16  =>  Data_16 : Word_Array (1 .. 10);
        when Format_32  =>  Data_32 : Long_Array (1 .. 5);
        end case;
    end record;

    type Mapping_Event_Record is
    record
        --        Window_Id     : Drawable;     -- unused
        Request       : Mapping_Request_Type;
        First_Keycode : Keycode; -- window first_keycode
        Count         : Keycode; -- defines range of change
    end record;

    type Extension_Event_Data is array (1..8) of X_Long_Integer ;
    
    type Event_Record (Kind : Event_Type := Enter_Notify);
    type Event is access Event_Record;

    type Event_Record (Kind : Event_Type := Enter_Notify) is
    record
        Serial      : X_Long_Integer;
        -- of last request processed by server
        Send_Event  : Boolean;
        -- true if this came from a Send_Event request
        Display_Id  : Display; -- Display the event was read from
        From_Window : Drawable;  -- "event" window it is reported relative to
        Extension   : Extension_Name ; -- server extension implied
        Event_Code  : Event_Code_Type ; -- wire event code
        case Kind is
        when Button_Press    | Button_Release    =>
            Button : Button_Event_Record;
        when Circulate_Notify     =>
            Circulate : Circulate_Event_Record;
        when Circulate_Request    =>
            Circulate_Reqst : Circulate_Request_Event_Record;
        when Client_Message  =>
            Client : Client_Message_Event_Record;
        when Colormap_Notify =>
            Colormap : Colormap_Event_Record;
        when Configure_Notify     =>
            Configure : Configure_Event_Record;
        when Configure_Request    =>
            Configure_Reqst : Configure_Request_Event_Record;
        when Create_Notify   =>
            Create_Window : Create_Window_Event_Record;
        when Destroy_Notify  =>
            Destroy_Window : Destroy_Window_Event_Record;
        when Enter_Notify    | Leave_Notify =>
            Crossing : Crossing_Event_Record;
        when Error  =>
            Error_Info : Error_Event_Record ;
        when Expose     =>
            Expose_Notify : Expose_Event_Record;
        when Focus_In   | Focus_Out    =>
            Focus : Focus_Change_Event_Record;
        when Graphics_Expose =>
            Graphics : Graphics_Expose_Event_Record;
        when Gravity_Notify  =>
            Gravity : Gravity_Event_Record;
        when Key_Press  | Key_Release  =>
            Key : Key_Event_Record;
        when Keymap_Notify   =>
            Keymap : Keymap_Event_Record;
        when Map_Notify =>
            Map : Map_Event_Record;
        when Mapping_Notify  =>
            Mapping : Mapping_Event_Record;
        when Map_Request     =>
            Map_Reqst : Map_Request_Event_Record;
        when Motion_Notify   =>
            Motion : Motion_Event_Record;
        when No_Expose  =>
            Noexpose : No_Expose_Event_Record;
        when Property_Notify =>
            Property : Property_Event_Record;
        when Reparent_Notify =>
            Reparent : Reparent_Event_Record;
        when Resize_Request  =>
            Resize_Reqst : Resize_Request_Event_Record;
        when Selection_Clear =>
            Selection_Clr : Selection_Clear_Event_Record;
        when Selection_Notify     =>
            Selection : Selection_Event_Record;
        when Selection_Request    =>
            Selection_Reqst : Selection_Request_Event_Record;
        when Unmap_Notify    =>
            Unmap : Unmap_Event_Record;
        when Visibility_Notify    =>
            Visibility : Visibility_Event_Record;
        when others     =>
            Any : Extension_Event_Data;
        end case;
    end record;

    -- per character font metric information.

    type Character_Record is
    record
        Left_Bearing  : Pixels := 0; -- origin to left edge of raster
        Right_Bearing : Pixels := 0; -- origin to right edge of raster
        Width         : Pixels := 0; -- advance to next char's origin
        Ascent        : Pixels := 0; -- baseline to top edge of raster
        Descent       : Pixels := 0; -- baseline to bottom edge of raster
        Attributes    : Pixels := 0; -- per char flags (not predefined)
    end record;
    for Character_Record use
    record
        Left_Bearing   at 0 * X_32_Bits range  0 .. 15 ;
        Right_Bearing  at 0 * X_32_Bits range 16 .. 31 ;
        Width          at 1 * X_32_Bits range  0 .. 15 ;
        Ascent         at 1 * X_32_Bits range 16 .. 31 ;
        Descent        at 2 * X_32_Bits range  0 .. 15 ;
        Attributes     at 2 * X_32_Bits range 16 .. 31 ;
    end record;

    type Character_Array is array (X_Character range <>) of
        Character_Record;
    type Character_List ( Length : X_Character := 0 ) is
    record
        Characters : Character_Array ( 1 .. Length ) ;
    end record ;

    -- To allow arbitrary information with fonts,
    -- there are additional properties returned.
    type Font_Property_Record is
    record
        Name    : Atom;
        Value   : X_Long_Integer;
    end record;
    type Font_Property_Array is
        array (X_Natural range <>) of Font_Property_Record;
    type Font_Property_List ( Length : X_Short_Natural := 0 ) is
    record
        Properties : Font_Property_Array ( 1 .. Length ) ;
    end record ;

    type Font_Record is
    record
        Font_Id      : Font;  -- Font id for this font
        Direction    : Font_Direction; --hint about dir. font is painted
        First_Char   : X_Character;
        Last_Char    : X_Character;
        First_Row    : X_Character;
        Last_Row     : X_Character;
        All_Exist    : Boolean; -- flag if all chars have non-zero size
        Default_Char : Character_2b; -- char to print for undefined chars
        Properties   : Font_Property_List;-- list of additional properties
        Min_Bounds   : Character_Record;-- bounds over all existing
        Max_Bounds   : Character_Record;-- characters
        Per_Char     : Character_List;  -- first_char to last_char info
        Ascent       : Pixels;  -- log. extent above baseline for spacing
        Descent      : Pixels;  -- log. descent below baseline for spacing
    end record;
    type Font_Record_Pointer is access Font_Record;

    type Font_Record_Array is array (X_Natural range <>) of
        Font_Record_Pointer ;
    type Font_Record_List ( Length : X_Short_Natural := 0 ) is
    record
        Font_Records : Font_Record_Array ( 1 .. Length ) ;
    end record ;

    type Color_Map_Array is array (X_Natural range <>) of Color_Map;
    type Color_Map_List ( Length : X_Natural := 0 ) is
    record
        Color_Maps : Color_Map_Array ( 1 .. Length ) ;
    end record ;

    type Color_Flag is
    record
        Red   : Boolean ;
        Green : Boolean ;
        Blue  : Boolean ;
    end record ;
    
    for Color_Flag use
    record
        Red       at 0 range 0 .. 0 ;
        Green     at 0 range 1 .. 1 ;
        Blue      at 0 range 2 .. 2 ;
    end record ;

    Do_Red : constant Color_Flag := Color_Flag'( Red    => True,
                                                 others => False);
    Do_Green : constant Color_Flag := Color_Flag'( Green  => True,
                                                   others => False);
    Do_Blue : constant Color_Flag := Color_Flag'( Blue   => True,
                                                  others => False);

    -- Data structure used by color operations
    type Color_Record is
    record
        Value : Pixel_Value;
        Red   : Rgb_Value_Type;
        Green : Rgb_Value_Type;
        Blue  : Rgb_Value_Type;
        Flags : Color_Flag;
    end record;

    for Color_Record use
    record at mod 2 ;
        Value   at 0 * X_32_Bits range  0 .. 31 ;
        Red     at 1 * X_32_Bits range  0 .. 15 ;
        Green   at 1 * X_32_Bits range 16 .. 31 ;
        Blue    at 2 * X_32_Bits range  0 .. 15 ;
        Flags   at 2 * X_32_Bits range 16 .. 18 ;
    end record;

    type Color_Array is array (X_Natural range <>) of Color_Record;

    type Pixel_Array is array (X_Natural range <>) of Pixel_Value;
    type Pixel_List ( Length : X_Natural := 0 ) is
    record
        Pixels : Pixel_Array ( 1 .. Length ) ;
    end record ;

    type Standard_Colormap is
    record
        Map_Id     : Color_Map;
        Red_Max    : Pixel_Value;
        Red_Mult   : Pixel_Value;
        Green_Max  : Pixel_Value;
        Green_Mult : Pixel_Value;
        Blue_Max   : Pixel_Value;
        Blue_Mult  : Pixel_Value;
        Base_Pixel : Pixel_Value;
        Visual_Id  : X_Id ;
        Kill_Id    : Resource ;
    end record;

    type Standard_Colormap_Array is array (X_Natural range <>) of
        Standard_Colormap ;
    type Standard_Colormap_List ( Length : X_Short_Natural := 0 ) is
    record
        Maps : Standard_Colormap_Array ( 1 .. Length ) ;
    end record ;

    type Text_Item is
    record
        Characters   : String_Record;
        String_Delta : Pixels; -- delta between strings 
        Font_Id      : Font := Null_Font;
        -- font to print it in, None don't change
    end record;
    type Text_Item_Array is array (X_Natural range <>) of Text_Item;
    type Text_Item_List ( Length : X_Natural := 0 ) is
    record
        Text_Items : Text_Item_Array ( 1 .. Length ) ;
    end record ;

    type Text_Item_8 is
    record
        Characters   : String_Record_8;
        String_Delta : Pixels; -- delta between strings 
        Font_Id      : Font := Null_Font ;
        -- font to print it in, None don't change
    end record;
    type Text_Item_8_Array is array (X_Natural range <>) of Text_Item_8;
    type Text_Item_8_List ( Length : X_Natural := 0 ) is
    record
        Text_Item_8s : Text_Item_8_Array ( 1 .. Length ) ;
    end record ;

    type Text_Item_16 is
    record
        Characters   : String_Record_16;
        String_Delta : Pixels; -- delta between strings 
        Font_Id      : Font := Null_Font;
        -- font to print it in, None don't change
    end record;
    type Text_Item_16_Array is array (X_Natural range <>) of Text_Item_16;
    type Text_Item_16_List ( Length : X_Natural := 0 ) is
    record
        Text_Item_16s : Text_Item_16_Array ( 1 .. Length ) ;
    end record ;

    type Gc_Value_Record is
    record
        Logical_Operation  : Gx_Function_Code    := Gx_Copy;
        Plane              : Plane_Mask          := All_Planes;
        Foreground         : Pixel_Value         := 0;
        -- foreground pixel
        Background         : Pixel_Value         := 1;
        -- background pixel
        Line_Width         : Pixels              := 0;
        Line_Style         : Line_Style_Type     := Line_Solid;
        Cap_Style          : Cap_Style_Type      := Cap_Butt;
        Join_Style         : Join_Style_Type     := Join_Miter;
        Fill_Style         : Fill_Style_Type     := Fill_Solid;
        Fill_Rule          : Fill_Rule_Type      := Even_Odd_Rule;
        Tile               : Pixmap;
        -- tile pixmap for tiling operations
        Stipple            : Pixmap;
        -- stipple 1 plane pixmap for stipping  
        Ts_X_Origin        : Coordinate          := 0;
        -- offset for tile or
        Ts_Y_Origin        : Coordinate          := 0;
        -- stipple operations
        Font_Id            : Font                := Null_Font;
        -- default font for text operations
        Subwindow_Mode     : Subwindow_Mode_Type := Clip_By_Children;
        Graphics_Exposures : Boolean             := True;
        Clip_X_Origin      : Coordinate          := 0;
        Clip_Y_Origin      : Coordinate          := 0;
        Clip_Mask          : Pixmap;
        -- bitmap clipping; other calls for rects
        Dash_Offset        : Pixels              := 0;
        -- patterned/dashed
        Dashes             : Short_Natural       := 4;
        -- line information
        Arc_Mode           : Arc_Mode_Type       := Arc_Pie_Slice;
    end record;

    -- Data structure for host setting/getting routines.
    --
    type Host_Name_Record ( Length : Short_Natural := 0 ) is
    record
        Family : Protocol_Family ;
        Name   : String (1 .. Length) ; -- Host name
    end record ;

    type Host_Array is array (Short_Natural range <>)
        of Host_Name_Record;
    type Host_List ( Length : Short_Natural := 0 ) is
    record
        Hosts : Host_Array ( 1 .. Length ) ;
    end record ;

    type Keycode_Array is array (X_Natural range <>) of Keycode;

    subtype Key_Sym is X_Key_Symbols.Symbol ;
    type Key_Sym_Array is array (Natural range <>) of Key_Sym;
    type Key_Sym_List ( Length : Short_Natural := 0 ) is
    record
        Syms : Key_Sym_Array ( 1 .. Length ) ;
    end record ;
    
    type Keyboard_Encoding_Array is array ( Keycode range <>,
                                            Natural range <> ) of Key_Sym;
    
    type Keyboard_Encoding_List ( First       : Keycode := 8 ;
                                  Last        : Keycode := 8 ;
                                  Per_Code    : Thin_Natural := 0 ) is
    record
        Syms : Keyboard_Encoding_Array (First .. Last, 1 .. Per_Code) ;
    end record ;

    -- Data structure for Change_Keyboard_Control
    type Keyboard_Control_Record is
    record
        Key_Click_Percent : Percent_Type;
        Bell_Volume       : Bell_Volume_Type;
        Bell_Pitch        : Hertz_Type;
        Bell_Duration     : Milliseconds_Type;
        Led               : Led_Type;
        Led_Mode          : Led_Mode_Type;
        Key               : Keycode;
        Auto_Repeat_Mode  : Auto_Repeat_Mode_Type;
    end record;

    -- Data structure for Get_Keyboard_Control
    type Keyboard_State_Record is
    record
        Key_Click_Percent  : Percent_Type;
        Bell_Volume        : Bell_Volume_Type;
        Bell_Pitch         : Hertz_Type;
        Bell_Duration      : Milliseconds_Type;
        Led_Mask           : Led_Mask_Type;
        Global_Auto_Repeat : Auto_Repeat_Mode_Type;
        Auto_Repeats       : Key_Vector_Mask_Type;
    end record;

    type Modifier_Keymap_Array is array
        ( Modifier_Type range <>, Thin_Natural range <> ) of Keycode ;
    
    type Modifier_Keymap ( Max_Keys_Per_Mod : Thin_Natural := 0 ) is
    record -- The server's max number of keys per modifier
        Modifiers   : Modifier_Keymap_Array ( Modifier_Type,
                                              1 .. Max_Keys_Per_Mod);
    end record;  -- An 8 by max_key_per_mod array of modifiers

    -- Data structure for Get_Motion_Events.
    type Time_Coord_Record is
    record
        Time_Stamp : Time;
        XY         : Point;
    end record;
    type Time_Coord_Array is array (X_Natural range <>)
        of Time_Coord_Record;
    type Time_Coord_List ( Length : X_Natural := 0 ) is
    record
        Time_Coords : Time_Coord_Array ( 1 .. Length ) ;
    end record ;

    type Compose_Status_Record is
    record
        Compose_Pointer : String_Record;
        Chars_Matched   : X_Natural;
    end record;

    
private

    type Atom is  range 0 .. (2 ** 29) - 1;
    type Region is  range 0 .. (2 ** 29) - 1;
    
    type Null_Record is
    record
        null ;
    end record ;
    
    type Display is access Null_Record ;
    
    for Extension_Event_Data'size use 8 * 32 ;
    
    type Graphic_Context is access Null_Record;

    type Image is access Null_Record ;

    type Screen is access Null_Record ;

    type Visual is access Null_Record ;
    

    Null_Display              : constant Display         := null;
    Null_Graphic_Context      : constant Graphic_Context := null;
    Null_Region               : constant Region          := Region (0);
    Image_Context             : constant Image           := null;
    Null_Visual               : constant Visual          := null;
    Copy_Visual_From_Parent   : constant Visual          := null;

    --
    -- Predefined Standard Properties
    --
    Null_Atom              : constant Atom := Atom (0);
    Xa_Primary             : constant Atom := 1;
    Xa_Secondary           : constant Atom := 2;
    Xa_Arc                 : constant Atom := 3;
    Xa_Atom                : constant Atom := 4;
    Xa_Bitmap              : constant Atom := 5;
    Xa_Cardinal            : constant Atom := 6;
    Xa_Colormap            : constant Atom := 7;
    Xa_Cursor              : constant Atom := 8;
    Xa_Cut_Buffer_0        : constant Atom := 9;
    Xa_Cut_Buffer_1        : constant Atom := 10;
    Xa_Cut_Buffer_2        : constant Atom := 11;
    Xa_Cut_Buffer_3        : constant Atom := 12;
    Xa_Cut_Buffer_4        : constant Atom := 13;
    Xa_Cut_Buffer_5        : constant Atom := 14;
    Xa_Cut_Buffer_6        : constant Atom := 15;
    Xa_Cut_Buffer_7        : constant Atom := 16;
    Xa_Drawable            : constant Atom := 17;
    Xa_Font                : constant Atom := 18;
    Xa_Integer             : constant Atom := 19;
    Xa_Pixmap              : constant Atom := 20;
    Xa_Point               : constant Atom := 21;
    Xa_Rectangle           : constant Atom := 22;
    Xa_Resource_Manager    : constant Atom := 23;
    Xa_Rgb_Color_Map       : constant Atom := 24;
    Xa_Rgb_Best_Map        : constant Atom := 25;
    Xa_Rgb_Blue_Map        : constant Atom := 26;
    Xa_Rgb_Default_Map     : constant Atom := 27;
    Xa_Rgb_Gray_Map        : constant Atom := 28;
    Xa_Rgb_Green_Map       : constant Atom := 29;
    Xa_Rgb_Red_Map         : constant Atom := 30;
    Xa_String              : constant Atom := 31;
    Xa_Visual_Id           : constant Atom := 32;
    Xa_Window              : constant Atom := 33;

    --
    -- Window Manager Standard Properties
    --
    Xa_Wm_Command          : constant Atom := 34;
    Xa_Wm_Hints            : constant Atom := 35;
    Xa_Wm_Client_Machine   : constant Atom := 36;
    Xa_Wm_Icon_Name        : constant Atom := 37;
    Xa_Wm_Icon_Size        : constant Atom := 38;
    Xa_Wm_Name             : constant Atom := 39;
    Xa_Wm_Normal_Hints     : constant Atom := 40;
    Xa_Wm_Size_Hints       : constant Atom := 41;
    Xa_Wm_Zoom_Hints       : constant Atom := 42;
    Xa_Wm_Class            : constant Atom := 67;
    Xa_Wm_Transient_For    : constant Atom := 68;

    --
    -- Standard Font Properties
    --
    Xa_Min_Space           : constant Atom := 43;
    Xa_Norm_Space          : constant Atom := 44;
    Xa_Max_Space           : constant Atom := 45;
    Xa_End_Space           : constant Atom := 46;
    Xa_Superscript_X       : constant Atom := 47;
    Xa_Superscript_Y       : constant Atom := 48;
    Xa_Subscript_X         : constant Atom := 49;
    Xa_Subscript_Y         : constant Atom := 50;
    Xa_Underline_Position  : constant Atom := 51;
    Xa_Underline_Thickness : constant Atom := 52;
    Xa_Strikeout_Ascent    : constant Atom := 53;
    Xa_Strikeout_Descent   : constant Atom := 54;
    Xa_Italic_Angle        : constant Atom := 55;
    Xa_X_Height            : constant Atom := 56;
    Xa_Quad_Width          : constant Atom := 57;
    Xa_Weight              : constant Atom := 58;
    Xa_Point_Size          : constant Atom := 59;
    Xa_Resolution          : constant Atom := 60;
    Xa_Copyright           : constant Atom := 61;
    Xa_Notice              : constant Atom := 62;
    Xa_Font_Name           : constant Atom := 63;
    Xa_Family_Name         : constant Atom := 64;
    Xa_Full_Name           : constant Atom := 65;
    Xa_Cap_Height          : constant Atom := 66;

    Xa_Last_Predefined     : constant Atom := Xa_Wm_Transient_For;

end X_Types;
